//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import "FileGridViewController.h"
#import "ImageGridCell.h"
#import "ImageViewerViewController.h"
#import "WebViewController.h"
#import "PCResource+GridCell.h"
#import "UIViewController+RightBarButtonAction.h"
#import "UIViewController+Alert.h"
#import "UIViewController+Dismiss.h"
#import "PCAccountInfo+additions.h"
#import "PCResource+Info.h"
#import "UIViewController+Spinner.h"
#ifdef KEPLER_FILE_BROWSER
#import "TrashViewController.h"
#import "MetaDataViewController.h"
#import "MoveViewController.h"
#import "CopyViewController.h"
#endif
#import "UIViewController+ResourceActions.h"
#import "NSArray+Addition.h"
#import "ImageDownloadView.h"

#ifdef KEPLER_FILE_BROWSER
typedef enum {
    
    metadataGridViewType,
    trashGridViewType,
    copyGridViewType,
    moveGridViewType,
} GridViewType;
#endif

static NSMutableDictionary *thumbnailImageCache;

@interface FileGridViewController () <ImageGridCellDelegate>

@property (weak, nonatomic) IBOutlet UICollectionView *gridView;
@property (nonatomic, strong) UIRefreshControl *refreshControl;
@property (nonatomic, strong) NSString *emptyViewMessage;

#ifdef KEPLER_FILE_BROWSER
@property (nonatomic, assign) GridViewType gridViewType ;
#endif

@end

@implementation FileGridViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.refreshControl = [[UIRefreshControl alloc] init];
    [self.refreshControl addTarget:self action:@selector(reloadData) forControlEvents:UIControlEventValueChanged];
    [self.gridView addSubview:self.refreshControl];
    
    [self.gridView layoutIfNeeded];
    [self.gridView scrollToItemAtIndexPath:self.selectedIndexPath atScrollPosition:UICollectionViewScrollPositionTop animated:NO];

    
    self.gridView.alwaysBounceVertical = YES;
    
    self.emptyViewMessage = NSLocalizedString(@"Empty", nil);
    
#ifdef KEPLER_FILE_BROWSER
    
    if ([self isMemberOfClass:[MetaDataViewController class]]) {
        
        MetaDataViewController *metadataSelf = (MetaDataViewController *)self;
        
        self.gridViewType = metadataGridViewType;
        
        if (metadataSelf.rootResource) {
            
            [self addRightBarButtonItemWithActions:[NSNumber numberWithInt:uploadFileRightBarAction], [NSNumber numberWithInt:accountInfoRightBarAction], [NSNumber numberWithInt:createFolderRightBarAction], [NSNumber numberWithInt:searchRightBarAction], [NSNumber numberWithInt:signOutRightBarAction], nil];
        } else {
        
            [self addRightBarButtonItemWithActions:[NSNumber numberWithInt:accountInfoRightBarAction], [NSNumber numberWithInt:searchRightBarAction], [NSNumber numberWithInt:signOutRightBarAction], nil];
        }
    } else if ([self isMemberOfClass:[CopyViewController class]]) {
    
        self.gridViewType = copyGridViewType;
        [self addRightBarButtonItemWithActions:[NSNumber numberWithInt:createFolderRightBarAction], nil];
    } else if ([self isMemberOfClass:[MoveViewController class]]) {

        self.gridViewType = moveGridViewType;
        [self addRightBarButtonItemWithActions:[NSNumber numberWithInt:createFolderRightBarAction], nil];
    } else if ([self isMemberOfClass:[TrashViewController class]]) {
    
        self.gridViewType = trashGridViewType;
        [self addRightBarButtonItemWithActions:[NSNumber numberWithInt:emptyTrashRightBarAction],[NSNumber numberWithInt:signOutRightBarAction], nil];
    }

#endif
    
#ifdef HUBBLE_MEDIA_BROWSER
    [self addRightBarButtonItemWithActions:[NSNumber numberWithInt:searchRightBarAction], [NSNumber numberWithInt:accountInfoRightBarAction], [NSNumber numberWithInt:signOutRightBarAction], nil];
#endif

}

#pragma mark -
#pragma mark Private methods

- (void)reloadData {
    // abstract method
}


- (void)refreshView {

    [self.gridView reloadData];
    
    if (!self.dataSource.count) {
        [self showOKAlertWithTitle:NSLocalizedString(@"Empty", nil)
                        andMessage:self.emptyViewMessage
                  attributedString:nil
                          okAction:nil];
    }
}

#pragma mark -
#pragma mark CollectionView datasource methods

- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section {
    return self.dataSource.count;
}

- (BOOL)collectionView:(UICollectionView *)collectionView shouldSelectItemAtIndexPath:(NSIndexPath *)indexPath {
    
    BOOL enableCell = YES;
#ifdef KEPLER_FILE_BROWSER
    PCResource *resource = [self.dataSource objectAtIndex:indexPath.row];
    if ([[resource file] boolValue]) {
        switch (self.gridViewType) {
            case moveGridViewType:
            case copyGridViewType:
                enableCell = NO;
                break;
            default:
                break;
        }
    }
    
    UICollectionViewCell *cell = [collectionView cellForItemAtIndexPath:indexPath];
    [cell setSelected:YES];
#endif
    return enableCell;
}


- (UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath {

    ImageGridCell *cell = [self.gridView dequeueReusableCellWithReuseIdentifier:@"thumbnailCell" forIndexPath:indexPath];
    
    PCResource *resource = [self.dataSource objectAtIndex:indexPath.row];
    cell.gridCellDelegate = self;
    cell.titlelabel.text = resource.name;

    
    if ((resource.contentToken == cell.imageView.imageID) && cell.imageView.api) {
        // do nothing
    } else {
    
        [cell.imageView.api cancelAPI];
        
        [cell.spinner startAnimating];
        [resource setThumbnailForImageView:cell.imageView withCompletionHandler:^(BOOL didSetImage) {
            
            if (didSetImage) {
                [cell.spinner stopAnimating];
            }
        }];
    }
    return cell;
}


#pragma mark -
#pragma mark CollectionView delegate methods

- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath {

    PCResource *resource = [self.dataSource objectAtIndex:indexPath.row];
    
    if ([[resource file] boolValue]) {
        
        PCResource *file = (PCResource *)resource;
        if (file.fileType == imageFileType || file.fileType == musicFileType ||  file.fileType == videoFileType) {
            
            UIStoryboard *storyBoard = [UIStoryboard storyboardWithName:@"Main" bundle:nil];
            ImageViewerViewController *controller = (ImageViewerViewController*)[storyBoard instantiateViewControllerWithIdentifier: @"ImageViewerViewController"];
            controller.dataSource = self.dataSource;
            controller.hidesBottomBarWhenPushed = YES;
            controller.selectedIndex = indexPath.row;
            [self.navigationController pushViewController:controller animated:YES];
        } else if (file.fileType == documentFileType) {
            UIStoryboard *storyBoard = [UIStoryboard storyboardWithName:@"Main" bundle:nil];
            WebViewController *controller = (WebViewController*)[storyBoard instantiateViewControllerWithIdentifier: @"WebViewController"];
            controller.selectedResource = resource;
            controller.hidesBottomBarWhenPushed = YES;
            [self.navigationController pushViewController:controller animated:YES];
        } else if (file.fileType == unknowFileType) {
            [self showOKAlertWithTitle:NSLocalizedString(@"Un-Supported format", nil)
                            andMessage:NSLocalizedString(@"This file format is not supported now", nil)
                      attributedString:nil
                              okAction:nil];
        }
    } else {
        
#ifdef KEPLER_FILE_BROWSER
        UIStoryboard *storyBoard = [UIStoryboard storyboardWithName:@"Main" bundle:nil];
        MetaDataViewController *baseController = nil;
        switch (self.gridViewType) {
            case metadataGridViewType: {
                MetaDataViewController *controller = (MetaDataViewController*)[storyBoard instantiateViewControllerWithIdentifier: @"MetaDataViewController"];
                baseController = controller;
            }
                break;
            case copyGridViewType: {
                CopyViewController *controller = (CopyViewController*)[storyBoard instantiateViewControllerWithIdentifier: @"CopyViewController"];
                __weak CopyViewController *typeCastedSelf = (CopyViewController *)self;
                controller.completionHandler = typeCastedSelf.completionHandler;
                controller.source = typeCastedSelf.source;
                baseController = controller;
            }
                break;
            case moveGridViewType: {
                MoveViewController *controller = (MoveViewController*)[storyBoard instantiateViewControllerWithIdentifier: @"MoveViewController"];
                __weak MoveViewController *typeCastedSelf = (MoveViewController *)self;
                controller.completionHandler = typeCastedSelf.completionHandler;
                controller.source = typeCastedSelf.source;
                baseController = controller;
            }
                break;
            default:
                break;
        }
        
        if (baseController) {
            
            baseController.rootResource = resource;
            [self.navigationController pushViewController:baseController animated:YES];
        }
#endif
    }
}


#pragma mark -
#pragma mark ImageGridCell delegate method

- (void)gridCellDidLongPressed:(ImageGridCell *)cell {
    NSIndexPath *indexPath = [self.gridView indexPathForCell:cell];
    PCResource *selectedResource = [self.dataSource objectAtIndex:indexPath.row];
#if KEPLER_FILE_BROWSER
    switch (self.gridViewType) {
        case trashGridViewType: {
            [self resourceSelected:selectedResource showActions:[NSNumber numberWithInt:restoreResourceAction], nil];
        }
            break;
        case metadataGridViewType: {
            if ([self isMemberOfClass:[MetaDataViewController class]]) {
                MetaDataViewController *metadataSelf = (MetaDataViewController *)self;
                    if (!metadataSelf.rootResource) {
                        break;
                    }
            }
            [self resourceSelected:selectedResource showActions:[NSNumber numberWithInt:copyResourceAction],
                 [NSNumber numberWithInt:moveResourceAction], [NSNumber numberWithInt:renameResourceAction], [NSNumber numberWithInt:deleteResourceAction], nil];
        }
            break;
        default:
            break;
    }
#endif
#if HUBBLE_MEDIA_BROWSER
    
    if (selectedResource.fileType == musicFileType) {
        [self resourceSelected:selectedResource showActions:[NSNumber numberWithInt:fileInfoResourceAction], [NSNumber numberWithInt:addToPlaylistResourceAction], nil];

    } else if (selectedResource.fileType == imageFileType || selectedResource.fileType == videoFileType) {
        [self resourceSelected:selectedResource showActions:[NSNumber numberWithInt:fileInfoResourceAction], [NSNumber numberWithInt:addToAlbumResourceAction], nil];
    }
    
    #endif

 
}

#pragma mark -
#pragma mark RightBarButtonAction callback

- (void)didBeginDoingRightBarButtonAction:(rightBarAction)action {
    [self showSpinner];
}


- (void)didFinishDoingRightBarButtonAction:(rightBarAction)action :(id)result :(id)response {

    [self hideSpinner];
    
    PCAPIQueryResponse *APIResponse = (PCAPIQueryResponse *)response;
    
    switch (APIResponse.urlResponse.statusCode) {
        case PCAPIResponseCodeResourceCreated:
        case PCAPIResponseCodeOK: {
        
            switch (action) {
                case accountInfoRightBarAction: {
                
                    PCAccountInfo *accountInfo = (PCAccountInfo *)result;
                    [self showOKAlertWithTitle:[self stringForRightBarAction:action]
                                    andMessage:[accountInfo humanReadableAccountInfo]
                              attributedString:nil
                                      okAction:nil];
                }
                    break;
#ifdef KEPLER_FILE_BROWSER
                case uploadFileRightBarAction:
                case createFolderRightBarAction: {
                    self.dataSource = [self.dataSource arrayByAddingObject:result andSortByKeyPath:nil ascending:NO];
                    [self refreshView];
                }
                    break;
#endif
                case refreshAccountBarAction: {
                
                    [self showOKAlertWithTitle:NSLocalizedString(@"Success", nil) andMessage:NSLocalizedString(@"Successfully refreshed account. Please pull to refresh the screen", nil) attributedString:nil okAction:nil];
                }
                    break;
                default:
                    break;
            }

        }
            break;
        case PCAPIResponseCodeUnauthorized:
            [self showUnAuthorizedAlert];
            break;
        case PCAPIResponseCodeServiceUnavailable:
            [self showServerUnavailableAlert];
            break;
        default:
            [self showOperationFailedAlert:[self stringForRightBarAction:action]];
            break;
    }    
}

- (id)dataSourceForRightBarButtonAction:(rightBarAction)action {
    return nil;
}

- (void)dealloc {
    
    for (ImageGridCell *cell in self.gridView.visibleCells) {
        
        [cell.imageView.api cancelAPI];
    }
}

#pragma mark -


- (void)didBeginDoingResourceAction:(ResourceAction)action {
    [self showSpinner];
}

- (void)didFinishDoingAction:(ResourceAction)action onResource:(id)resource result:(id)result response:(id)response {
    
    PCAPIQueryResponse *APIResponse = (PCAPIQueryResponse *)response;
    
    [self hideSpinner];
    
    switch (APIResponse.urlResponse.statusCode) {
        case PCAPIResponseCodeResourceCreated:
        case PCAPIResponseCodeNoContent:
        case PCAPIResponseCodeOK: {
            
            switch (action) {
#ifdef KEPLER_FILE_BROWSER

                case moveResourceAction:
                case copyResourceAction:
                    [self showSpinner];
                    [self reloadData];
                    break;
                case restoreResourceAction:
                case deleteResourceAction: {
                    
                    NSMutableArray *mutableDataSource = [NSMutableArray arrayWithArray:self.dataSource];
                    [mutableDataSource removeObject:resource];
                    self.dataSource = [NSArray arrayWithArray:mutableDataSource];
                    [self refreshView];
                }
                    break;
                case renameResourceAction: {
                    
                    self.dataSource = [self.dataSource arrayByReplacingObject:resource withObject:result andSortByKeyPath:nil ascending:NO];
                    [self refreshView];
                }
                    break;
#endif
#if HUBBLE_MEDIA_BROWSER
                case addToAlbumResourceAction:
                    case fileInfoResourceAction:
                    case addToPlaylistResourceAction:
                    break;
                case removeFromAlbumResourceAction:
                case removeFromPlaylistResourceAction: {
                
                    NSMutableArray *mutableDataSource = [NSMutableArray arrayWithArray:self.dataSource];
                    [mutableDataSource removeObject:resource];
                    self.dataSource = [NSArray arrayWithArray:mutableDataSource];
                    [self refreshView];

                }
                default:
                    break;
#endif
            }
        }
            break;
        case PCAPIResponseCodeUnauthorized:
            [self showUnAuthorizedAlert];
            break;
        case PCAPIResponseCodeServiceUnavailable:
            [self showServerUnavailableAlert];
            break;
        default:
            [self showOperationFailedAlert:[self stringForResourceAction:action]];
            break;
    }
}

@end
