//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import "PCResource+GridCell.h"
#import "PCResource+Info.h"
#import <UIKit/UIKit.h>
#import "ImageDownloadView.h"

@implementation PCResource (GridCell)


- (UIImage *)defaultPic {
    
    if ([[self file] boolValue]) {
        
        PCResource *file = (PCResource *)self;
        
        UIImage *defaultImage = [UIImage imageNamed:@"DefaultPic"];
        
        if (file.fileType == imageFileType) {
            defaultImage = [UIImage imageNamed:@"DefaultPic"];
        } else if (file.fileType == videoFileType) {
            defaultImage = [UIImage imageNamed:@"VideoDefault"];
        } else if (file.fileType == musicFileType) {
            defaultImage = [UIImage imageNamed:@"MusicDefault"];
        }
        return defaultImage;
        
    } else {
        
        return [UIImage imageNamed:@"Folder"];
    }
}

- (void)setThumbnailForImageView:(ImageDownloadView *)imageView withCompletionHandler:(void (^)(BOOL didSetImage))completion {
    
    if ([[self file] boolValue]) {
        
        PCResource *file = (PCResource *)self;
        imageView.imageID = file.contentToken; // imageID needs to be set to encounter cell reusability feature
        
        UIImage *thumbnailImage = [UIImage imageWithContentsOfFile:[self thumbnailCachePath]];
        
        if (thumbnailImage) {
            
            imageView.image = thumbnailImage;
            completion(YES);
        } else {
            
            imageView.image = nil;
            
            
            PCThumbnailAPIParams *params = [[PCThumbnailAPIParams alloc] init];
            params.th = 320;
            params.tw = 320;
            params.contentToken = file.contentToken;
            
            
            __weak PCResource *blocksafeSelf = self;
            
            PCAPI *thumbnailAPI = [[PCAPISDK sharedInstance] downloadThumbnail:params withCompletionHandler:^(PCAPIQueryResponse *response) {
                
                NSData *fileData = [NSData dataWithContentsOfURL:response.downloadedURL];
                if (fileData) {
                    [fileData writeToFile:[blocksafeSelf thumbnailCachePath] atomically:YES];
                }
                
                if ([imageView.imageID isEqualToString:blocksafeSelf.contentToken]) { // cell is still representing same resource
                    
                    
                    if (fileData) {
                        
                        UIImage *thumbImage = [UIImage imageWithContentsOfFile:[blocksafeSelf thumbnailCachePath]];
                        if (thumbImage) {
                            
                            imageView.image = thumbImage;
                            completion(YES);
                        } else {
                            
                            imageView.image = [blocksafeSelf defaultPic];
                            completion(YES);
                        }
                        
                    } else {
                        
                        imageView.image = [blocksafeSelf defaultPic];
                        completion(YES);
                    }
                } else {
                
                    completion(NO);
                }
            }];
            
            imageView.api = thumbnailAPI;
        }
        
    } else {
        
        imageView.imageID = nil;
        imageView.image = [self defaultPic];
        completion(YES);
    }
}

- (void)setFullResolutionImageView:(ImageDownloadView *)imageView withProgressHandler:(void (^)(float progress))progress andCompletionHandler:(void (^)(BOOL didSetImage))completion {
    
    if (self.fileType == imageFileType) {
        
        PCResource *file = (PCResource *)self;
        imageView.imageID = file.contentToken; // imageID needs to be set to encounter cell reusability feature
        
        UIImage *image = [[UIImage alloc] initWithContentsOfFile:[self fileCachePath]];
        
        if (image) {
            
            imageView.image = image;
            completion(YES);
        } else {
            
            imageView.image = nil;
            
            PCFilesAPIParams *params = [[PCFilesAPIParams alloc] init];
            params.path = self.fullPath;
            
            __weak PCResource *blocksafeSelf = self;
            
            PCAPI *filesAPI = [[PCAPISDK sharedInstance] downloadFile:params
                                                             progress:^(long long totalBytesDownloaded, long long totalBytesToBeDownloaded) {
                                                                 
                                                                 progress((float)totalBytesDownloaded/[[self size] floatValue]);
                                                                 
                                                             } withCompletionHandler:^(PCAPIQueryResponse *response) {
                                                                 NSData *fileData = [NSData dataWithContentsOfURL:response.downloadedURL];
                                                                 
                                                                 if (fileData) {
                                                                     
                                                                     [fileData writeToFile:[self fileCachePath] atomically:YES];
                                                                 }
                                                                 
                                                                 if ([imageView.imageID isEqualToString:blocksafeSelf.contentToken]) { // cell is still representing same resource
                                                                     
                                                                     
                                                                     if (fileData) {
                                                                         
                                                                         UIImage *fullResImage = [UIImage imageWithContentsOfFile:[blocksafeSelf fileCachePath]];
                                                                         if (fullResImage) {
                                                                             
                                                                             imageView.image = fullResImage;
                                                                             completion(YES);
                                                                         } else {
                                                                             
                                                                             imageView.image = [blocksafeSelf defaultPic];
                                                                             completion(YES);
                                                                         }
                                                                         
                                                                     } else {
                                                                         
                                                                         imageView.image = [blocksafeSelf defaultPic];
                                                                         completion(YES);
                                                                     }
                                                                 } else {
                                                                 
                                                                     completion(NO);
                                                                 }
                                                             }];
            
            imageView.api = filesAPI;
            
        }
    } else {
        
        imageView.imageID = nil;
        imageView.image = [self defaultPic];
        completion(YES);
    }
}

@end
