//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import "CustomDatePickerViewController.h"
#import "CustomDatePicker.h"

static const float duration = .25f;
static const float delay = 0.f;

@interface CustomDatePickerViewController ()

@property (nonatomic, weak) CustomDatePicker *customDatePickerView;

@end

@implementation CustomDatePickerViewController

-(void)loadView {
   
    self.view = [[UIView alloc] initWithFrame:[[UIScreen mainScreen] bounds]];
}

-(void)showPickerView:(CustomDatePicker *)customDatePickerView completion:(void (^)(void))completion {
    //  Getting topMost ViewController
    UIViewController *topController = [[[UIApplication sharedApplication] keyWindow] rootViewController];
    while ([topController presentedViewController])	topController = [topController presentedViewController];
    
    _customDatePickerView = customDatePickerView;
    
    __block CGRect customPickerViewFrame = customDatePickerView.frame;
    
    {
        customPickerViewFrame.origin.y = self.view.bounds.size.height;
        customDatePickerView.frame = customPickerViewFrame;
        [self.view addSubview:customDatePickerView];
        
    }
    //Adding self.view to topMostController.view and adding self as childViewController to topMostController
    {
        self.view.frame = CGRectMake(0, 0, topController.view.bounds.size.width, topController.view.bounds.size.height);
        self.view.autoresizingMask = UIViewAutoresizingFlexibleHeight|UIViewAutoresizingFlexibleWidth;
        [topController addChildViewController: self];
        [topController.view addSubview: self.view];
        
    }
    
    __weak CustomDatePickerViewController *blocksafeSelf = self;
    
    //Sliding up the pickerView with animation
    [UIView animateWithDuration:duration delay:delay options:UIViewAnimationOptionBeginFromCurrentState|7<<16 animations:^{
        
        customPickerViewFrame.origin.y = blocksafeSelf.view.bounds.size.height-customPickerViewFrame.size.height;
        customDatePickerView.frame = customPickerViewFrame;
        blocksafeSelf.view.backgroundColor = [UIColor colorWithWhite:0.0f alpha:0.5f];
        
    } completion:^(BOOL finished) {
        if (completion) {
            completion();
        }
    }];
}

-(void)dismissWithCompletion:(void (^)(void))completion
{
    //Sliding down the pickerView with animation.
    __weak CustomDatePickerViewController *blocksafeSelf = self;
    
    [UIView animateWithDuration:duration delay:delay options:UIViewAnimationOptionBeginFromCurrentState|7<<16 animations:^{
    
        CGRect customPickerViewFrame = blocksafeSelf.customDatePickerView.frame;
        customPickerViewFrame.origin.y = blocksafeSelf.view.bounds.size.height;
        blocksafeSelf.customDatePickerView.frame = customPickerViewFrame;
        
        blocksafeSelf.view.backgroundColor = [UIColor clearColor];
        blocksafeSelf.view.alpha = 1.0f;
        
    } completion:^(BOOL finished) {
        
        //Removing pickerView from self.view
        [blocksafeSelf.customDatePickerView removeFromSuperview];
        
        //Removing self.view from topMostController.view and removing self as childViewController from topMostController
        [blocksafeSelf willMoveToParentViewController:nil];
        [blocksafeSelf.view removeFromSuperview];
        [blocksafeSelf removeFromParentViewController];
        
        if (completion) completion();
    }];
}

@end
