//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import "SearchConfigurationViewController.h"
#import "CustomDatePicker.h"
#import "SearchResultsViewController.h"
#import "UIViewController+Alert.h"
#import "UIViewController+RightBarButtonAction.h"
#import "UIViewController+Spinner.h"
#import "UIViewController+Dismiss.h"
#import "PCAccountInfo+additions.h"

typedef enum {
    
    documents   = 1 << 0,
    photos      = 1 << 1,
    videos      = 1 << 2,
    music       = 1 << 3,
    all         = documents | photos | videos | music,
} media;

enum {

    fromDateCalender,
    toDateCalender
};

@interface SearchConfigurationViewController () <UITextFieldDelegate>

@property (nonatomic, assign) media selectedMedia;
@property (weak, nonatomic) IBOutlet UILabel *lastModifiedLabel;
@property (weak, nonatomic) IBOutlet UILabel *lookInLabel;
@property (nonatomic, copy) NSString *startDateString;
@property (nonatomic, strong) NSDate *startDate;
@property (nonatomic, copy) NSString *endDateString;
@property (nonatomic, strong) NSDate *endDate;
@property (weak, nonatomic) IBOutlet UITextField *nameField;
@property (weak, nonatomic) IBOutlet UITextField *startDateField;
@property (weak, nonatomic) IBOutlet UITextField *endDateField;
@property (nonatomic, strong) NSDateFormatter *apiDateFormatter;
@property (nonatomic, strong) NSDateFormatter *dateFormatterForDisplay;
@property (weak, nonatomic) IBOutlet UIButton *documentsButton;
@property (weak, nonatomic) IBOutlet UIButton *photosButton;
@property (weak, nonatomic) IBOutlet UIButton *videosButton;
@property (weak, nonatomic) IBOutlet UIButton *musicButton;
@property (weak, nonatomic) IBOutlet UIButton *allButton;
@property (weak, nonatomic) IBOutlet UIButton *advancedSearchButton;
@property (weak, nonatomic) IBOutlet UIButton *nameSearchButton;


- (IBAction)lookInOptionSelected:(id)sender;
- (IBAction)calenderSelected:(id)sender;
- (IBAction)searchByName:(id)sender;
- (IBAction)advancedSearch:(id)sender;

@end

@implementation SearchConfigurationViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.navigationItem.title = NSLocalizedString(@"Search", nil);
    
    self.selectedMedia = all;
    self.startDate = [NSDate date];
    self.endDate = [NSDate date];
    [self addRightBarButtonItemWithActions:[NSNumber numberWithInt:accountInfoRightBarAction], [NSNumber numberWithInt:refreshAccountBarAction], [NSNumber numberWithInt:signOutRightBarAction], nil];
    [self updateButtonState];
}

#pragma mark -
#pragma mark Right button callback

- (void)didBeginDoingRightBarButtonAction:(rightBarAction)action {
    
    [self showSpinner];
}

- (void)didFinishDoingRightBarButtonAction:(rightBarAction)action :(id)result :(id)response {
    
    [self hideSpinner];
    
    PCAPIQueryResponse *APIResponse = (PCAPIQueryResponse *)response;
    
    switch (APIResponse.urlResponse.statusCode) {
        case PCAPIResponseCodeOK: {
            
            switch (action) {
                case accountInfoRightBarAction: {
                    
                    PCAccountInfo *accountInfo = (PCAccountInfo *)result;
                    [self showOKAlertWithTitle:[self stringForRightBarAction:action]
                                    andMessage:[accountInfo humanReadableAccountInfo]
                              attributedString:nil
                                      okAction:nil];
                }
                    break;
                case refreshAccountBarAction: {
                    
                    [self showOKAlertWithTitle:NSLocalizedString(@"Success", nil) andMessage:NSLocalizedString(@"Successfully refreshed account. Please pull to refresh the screen", nil) attributedString:nil okAction:nil];
                }
                    break;
                default:
                    break;
            }
            
        }
            break;
        case PCAPIResponseCodeUnauthorized:
            [self showUnAuthorizedAlert];
            break;
        case PCAPIResponseCodeServiceUnavailable:
            [self showServerUnavailableAlert];
            break;
        default:
            [self showOperationFailedAlert:[self stringForRightBarAction:action]];
            break;
    }    
}


#pragma mark -
#pragma mark Private methods

- (NSDate *)dateWithDate:(NSDate *)date Hour:(NSInteger)hour Minute:(NSInteger)minute Second:(NSInteger)second {
    NSCalendar *calendar = [NSCalendar currentCalendar];
    NSDateComponents *components = [calendar components:NSCalendarUnitDay | NSCalendarUnitMonth | NSCalendarUnitYear fromDate:date];
    components.hour = hour;
    components.minute = minute;
    components.second = second;
    return [calendar dateFromComponents:components];
}

- (NSDateFormatter *)apiDateFormatter {
    if (!_apiDateFormatter) {
        NSDateFormatter *dateFormat = [[NSDateFormatter alloc] init];
        [dateFormat setDateFormat:@"yyyy-MM-dd'T'HH:mm:ss.SSS'Z'"];
        _apiDateFormatter = dateFormat;
    }
    return _apiDateFormatter;
}

- (NSDateFormatter *)dateFormatterForDisplay {
    if (!_dateFormatterForDisplay) {
        NSDateFormatter *dateFormat = [[NSDateFormatter alloc] init];
        [dateFormat setDateFormat:@"dd-MM-yyyy'"];
        _dateFormatterForDisplay = dateFormat;
    }
    return _dateFormatterForDisplay;
}

- (void)showFromDatePicker {
    
    __weak SearchConfigurationViewController *blocksafeSelf = self;
    
    CustomDatePicker *startCustomDatePicker = [[CustomDatePicker alloc] initWithTitle:(NSString *)@"Select Start Date" titleFont:[UIFont boldSystemFontOfSize:16.0] datePickerMode:UIDatePickerModeDate selectedDate:blocksafeSelf.startDate doneBlock:^(CustomDatePicker *picker, id selectedDate) {
       
        NSDate *beginningOfDate = [blocksafeSelf dateWithDate:selectedDate Hour:0 Minute:0 Second:0];
        blocksafeSelf.startDate = beginningOfDate;
        blocksafeSelf.startDateString = [[blocksafeSelf apiDateFormatter] stringFromDate:beginningOfDate];
        blocksafeSelf.startDateField.text = [[blocksafeSelf dateFormatterForDisplay] stringFromDate:beginningOfDate];
        
    } cancelBlock:^(CustomDatePicker *picker) {
        
        blocksafeSelf.startDate = nil;
        blocksafeSelf.startDateString = nil;
        blocksafeSelf.startDateField.text = nil;
        
    }];
    [startCustomDatePicker showPickerView];
}

- (void)showEndDatePicker {
    
    __weak SearchConfigurationViewController *blocksafeSelf = self;
    
    CustomDatePicker *endCustomDatePicker = [[CustomDatePicker alloc] initWithTitle:@"Select End Date" titleFont:[UIFont boldSystemFontOfSize:16.0] datePickerMode:UIDatePickerModeDate selectedDate:blocksafeSelf.endDate doneBlock:^(CustomDatePicker *picker, id selectedDate) {
       
        NSDate *beginningOfDate = [blocksafeSelf dateWithDate:selectedDate Hour:23 Minute:59 Second:59];
        blocksafeSelf.endDate = beginningOfDate;
        blocksafeSelf.endDateString = [[blocksafeSelf apiDateFormatter] stringFromDate:beginningOfDate];
        blocksafeSelf.endDateField.text = [[blocksafeSelf dateFormatterForDisplay] stringFromDate:beginningOfDate];
        
    } cancelBlock:^(CustomDatePicker *picker) {
        
        blocksafeSelf.endDate = nil;
        blocksafeSelf.endDateString = nil;
        blocksafeSelf.endDateField.text = nil;
        
    }];
    [endCustomDatePicker showPickerView];
}


- (BOOL)validateDate {

    if (self.startDate && self.endDate) {
        if ([self.endDate compare:self.startDate] == NSOrderedDescending) {
            return YES;
        } else {
            return NO;
        }
    } else if (self.startDate) {
        return NO;
    } else if (self.endDate) {
        return NO;
    } else {
        return YES;
    }
}

- (void)searchCompletion:(NSArray *)files {
    if (files.count) {
        UIStoryboard *storyBoard = [UIStoryboard storyboardWithName:@"Main" bundle:nil];
        SearchResultsViewController *controller = (SearchResultsViewController*)[storyBoard instantiateViewControllerWithIdentifier: @"SearchResultsViewController"];
        controller.dataSource = files;
        [self.navigationController pushViewController:controller animated:YES];
        
    } else {
        [self showOKAlertWithTitle:NSLocalizedString(@"No Search Results", nil)
                                 andMessage:NSLocalizedString(@"No Results found. Please modify your search and try again", nil)
                           attributedString:nil
                                   okAction:nil];
    }
}

- (void)doSearch:(PCSearchAPIParams *)params {
    
    __weak SearchConfigurationViewController *blocksafeSelf = self;
    
    self.advancedSearchButton.enabled = NO;
    self.nameSearchButton.enabled = NO;
    [self showSpinner];
    
    
    [[PCAPISDK sharedInstance] search:params withCompletionHandler:^(NSArray *results, PCAPIQueryResponse *response) {

        [blocksafeSelf hideSpinner];
        blocksafeSelf.advancedSearchButton.enabled = YES;
        blocksafeSelf.nameSearchButton.enabled = YES;
        
       
        switch (response.urlResponse.statusCode) {
            case PCAPIResponseCodeOK:
            case PCAPIResponseCodePreconditionFailed: {
                
                dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^(void) {
               
                    NSMutableArray *searchResultObjects = [NSMutableArray new];
                    for (NSManagedObjectID *objID in results) {
                        
                        PCResource *resource = [[PCResource alloc] initWithDBResourceID:objID];
                        [searchResultObjects addObject:resource];
                    }
                    
                    dispatch_async(dispatch_get_main_queue(), ^(void) {

                        [blocksafeSelf searchCompletion:searchResultObjects];
                    });
                });
            }
                break;
            case PCAPIResponseCodeUnauthorized:
                [blocksafeSelf showUnAuthorizedAlert];
                break;
            case PCAPIResponseCodeServiceUnavailable:
                [blocksafeSelf showServerUnavailableAlert];
                break;
            default:
                [blocksafeSelf showOperationFailedAlert:NSLocalizedString(@"Search", nil)];
                break;
        }
    }];
    
}

- (void)updateButtonState {
    self.documentsButton.selected = self.selectedMedia & documents;
    self.photosButton.selected = self.selectedMedia & photos;
    self.videosButton.selected = self.selectedMedia & videos;
    self.musicButton.selected = self.selectedMedia & music;
    self.allButton.selected = (self.selectedMedia & documents) && (self.selectedMedia & photos) && (self.selectedMedia & videos) && (self.selectedMedia & music);
}

#pragma mark -
#pragma mark Button action

- (IBAction)lookInOptionSelected:(id)sender {
    [self.nameField resignFirstResponder];
    UIButton *senderButton = (UIButton *)sender;
    senderButton.selected = !senderButton.selected;
    
    if (senderButton == self.allButton) {
        
        self.selectedMedia = senderButton.selected?all:~all;
        
    } else if (senderButton == self.documentsButton) {
        
        self.selectedMedia = self.selectedMedia ^ documents;
        
    } else if (senderButton == self.photosButton) {
        
        self.selectedMedia = self.selectedMedia ^ photos;
        
    } else if (senderButton == self.videosButton) {
        
        self.selectedMedia = self.selectedMedia ^ videos;
        
    } else if (senderButton == self.musicButton) {
        
        self.selectedMedia = self.selectedMedia ^ music;
        
    }
    
    [self updateButtonState];
}

- (IBAction)calenderSelected:(id)sender {

    [self.nameField resignFirstResponder];
    UIButton *senderButton = (UIButton *)sender;

    switch (senderButton.tag) {
        case fromDateCalender:
            [self showFromDatePicker];
            break;
        case toDateCalender:
            [self showEndDatePicker];
            break;
    }
}

- (IBAction)searchByName:(id)sender {
    
    [self.nameField resignFirstResponder];
    NSCharacterSet *whitespaceSet = [NSCharacterSet whitespaceCharacterSet];
    NSString *trimmed = [self.nameField.text stringByTrimmingCharactersInSet:whitespaceSet];

    if(trimmed.length == 0) {
        [self showOKAlertWithTitle:NSLocalizedString(@"Name Field Empty", nil) andMessage:NSLocalizedString(@"Please enter a name to be searched for", nil) attributedString:nil okAction:nil];
        
        return;
    }
    
    NSString *nameParamKeyValuePair = [NSString setValue:[NSString stringWithFormat:@"*%@*",self.nameField.text] forKey:@"name"];
    NSString *genreParamKeyValuePair = [NSString setValue:[NSString stringWithFormat:@"*%@*",self.nameField.text] forKey:@"genre"];
    NSString *artistParamKeyValuePair = [NSString setValue:[NSString stringWithFormat:@"*%@*",self.nameField.text] forKey:@"artist"];
    NSString *albumParamKeyValuePair = [NSString setValue:[NSString stringWithFormat:@"*%@*",self.nameField.text] forKey:@"album"];
    

    PCSearchAPIParams *params = [[PCSearchAPIParams alloc] init];
    
    params.query = [NSString doOR:[NSArray arrayWithObjects:nameParamKeyValuePair, genreParamKeyValuePair, artistParamKeyValuePair, albumParamKeyValuePair, nil]];
    
    [self doSearch:params];
}

- (IBAction)advancedSearch:(id)sender {

    if (![self validateDate]) {
        [self showOKAlertWithTitle:NSLocalizedString(@"Invalid Date", nil) andMessage:NSLocalizedString(@"Please enter a valid Start Date and End Date", nil)  attributedString:nil okAction:nil];
        
        return;
    }
    
    if (self.selectedMedia & all) { // bitwise AND: If non zero means something has been selected
        NSMutableArray *allKeyValuePairs = [NSMutableArray new];
        
        
        NSMutableArray *mediaTypeKeyValuePairArray = [NSMutableArray new];
        if (self.selectedMedia & photos) {
            NSString *photosKeyValuePair = [NSString setValue:@"image/*" forKey:@"contentType"];
            [mediaTypeKeyValuePairArray addObject:photosKeyValuePair];
        }
        
        if (self.selectedMedia & videos) {
            NSString *videosKeyValuePair = [NSString setValue:@"video/*" forKey:@"contentType"];
            [mediaTypeKeyValuePairArray addObject:videosKeyValuePair];
        }
        
        if (self.selectedMedia & music) {
            NSString *musicKeyValuePair = [NSString setValue:@"audio/*" forKey:@"contentType"];
            [mediaTypeKeyValuePairArray addObject:musicKeyValuePair];
        }
        
        if (self.selectedMedia & documents) {
            NSString *docsKeyValuePair = [NSString setValue:@"application/*" forKey:@"contentType"];
            [mediaTypeKeyValuePairArray addObject:docsKeyValuePair];
        }
        
        if (mediaTypeKeyValuePairArray.count) {
            NSString *ORQuery = [NSString doOR:[NSArray arrayWithArray:mediaTypeKeyValuePairArray]];
            [allKeyValuePairs addObject:ORQuery];
        }
        
        
        NSString *dateKeyValuePair;
        if (self.startDateString.length && self.endDateString.length) {
            dateKeyValuePair = [NSString setValue:[NSString stringWithFormat:@"[%@ TO %@]", self.startDateString, self.endDateString] forKey:@"versionCreated"];
            [allKeyValuePairs addObject:dateKeyValuePair];
        }
        
        
        PCSearchAPIParams *params = [[PCSearchAPIParams alloc] init];
        params.query = [NSString doAND:allKeyValuePairs];
        
        [self doSearch:params];
    } else { // bitwise AND: Zero means nothing has been selected
        [self showOKAlertWithTitle:NSLocalizedString(@"No Selection", nil) andMessage:NSLocalizedString(@"Please select atleast one media type to do a search", nil) attributedString:nil okAction:nil];
        
        return;
    }
    
}

#pragma mark -
#pragma mark UITextfield delegate

- (BOOL)textFieldShouldBeginEditing:(UITextField *)textField {
    if (textField == self.startDateField) {
        [self showFromDatePicker];
        return NO;
    } else if (textField == self.endDateField) {
        [self showEndDatePicker];
        return NO;
    }
    return YES;
}

- (BOOL)textFieldShouldReturn:(UITextField *)textField {
    [textField resignFirstResponder];
    return YES;
}

@end
