//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import "UIViewController+Dismiss.h"

@implementation UIViewController (Dismiss)

- (void)addCancelButton {
    UIBarButtonItem *cancelButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemCancel
                                                                                  target:self
                                                                                  action:@selector(dismissSelfOnly)];
    self.navigationItem.leftBarButtonItem = cancelButton;
}

- (void)dismissSelfRecursivelyAnimated:(BOOL)animated {

    /*
     * if viewController had been presented modally, self.presentingViewController will be non nil. in such case dismiss modal view controller.
     * Having dismissed modal viewController check and dismiss, if Modal ViewController had been presented from another Modal viewcontroller and so on. At the end of this stack, check if a navigation controller presented the modal view controllers. If YES pop those viewControllers as well to reach the rootViewController.
     */
    
    if (self.presentingViewController) {
        
        [self dismissViewControllerAnimated:animated completion:nil];
        [self.presentingViewController dismissSelfRecursivelyAnimated:animated];
        
    } else if (self.navigationController) {
        
        [self.navigationController popToRootViewControllerAnimated:animated];
    } else if ([self isKindOfClass:[UINavigationController class]]) {
    
        UINavigationController *navSelf = (UINavigationController *)self;
        [navSelf popToRootViewControllerAnimated:animated];
    }
}


- (void)dismissSelfOnly {
    if (self.presentingViewController) {
        [self dismissViewControllerAnimated:YES completion:nil];
    } else if (self.navigationController) {
        [self.navigationController popToRootViewControllerAnimated:YES];
    }
}

@end
