//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import <UIKit/UIKit.h>

@class PCPlaylist;

typedef enum {

    renamePlaylistAction,
    deletePlaylistAction,
} PlaylistAction;

@interface UIViewController (PlaylistActions)

- (NSString *)stringForPlaylistAction:(PlaylistAction)action;

/*!
 * call this method to trigger playlist actions like delete,rename etc.
 * @param selectedPlaylist playlist on which action needs to be performed
 */
- (void)showActionsForSelectedPlaylist:(PCPlaylist *)selectedPlaylist;

/*!
 * call this method to silently execute an action for any playlist
 */
- (void)executeAction:(PlaylistAction)action forPlaylist:(PCPlaylist *)playlist;

/*!
 * Implement this method in the viewController consuming this category class. This method can implement cusotm actions which needs to be implemented before performing a particular action. For eg. start a spinner
 * @param action the action that is going to be executed
 */
- (void)didBeginDoingPlaylistAction:(PlaylistAction)action;

/*!
 * Implement this method in the viewController consuming this category class. This method can implement cusotm actions which needs to be implemented after performing a particular action. For eg. stop a spinner, reload a gridview etc
 * @param action the action that is being executed
 * @param result result of action, if any
 * @param response response of the action, if any
 */
- (void)didFinishDoingAction:(PlaylistAction)action onPlaylist:(id)playlist result:(id)result response:(id)response;


@end
