//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import "UIViewController+PlaylistActions.h"
#import "UIViewController+Alert.h"
#import "PCPlaylist+Additions.h"

@implementation UIViewController (PlaylistActions)

- (NSString *)stringForPlaylistAction:(PlaylistAction)action {
    
    switch (action) {
        case renamePlaylistAction:
            return NSLocalizedString(@"Rename Playlist", nil);
            break;
        case deletePlaylistAction:
            return NSLocalizedString(@"Delete Playlist", nil);
            break;
        default:
            return nil;
            break;
    }
}

- (void)didBeginDoingPlaylistAction:(PlaylistAction)action {
    // abstract method
}

- (void)didFinishDoingAction:(PlaylistAction)action onPlaylist:(id)playlist result:(id)result response:(id)response {
    // abstract method
}

-(id) dataSourceForPlaylistAction:(PlaylistAction)action {
    return nil;
    // abstract method
}


#pragma mark -

- (void)executeAction:(PlaylistAction)action forPlaylist:(PCPlaylist *)playlist {

    switch (action) {
        case deletePlaylistAction:
            [self deletePlaylistActionCompletion:playlist];
            break;
        case renamePlaylistAction:
            [self renamePlaylistActionCompletion:playlist :nil];
            break;
        default:
            break;
    }
}

#pragma mark -
#pragma mark Cancel
- (void)addCancelAction:(UIAlertController *)alertController {
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Cancel", nil)
                    style:UIAlertActionStyleCancel
                andAction:nil];
}

#pragma mark -
#pragma mark Rename

- (void)renamePlaylist:(PCPlaylist *)selectedPlaylist :(NSString *)newName {
    __weak UIViewController *blocksafeSelf = self;

    PCEditPlaylistAPIParams *params = [[PCEditPlaylistAPIParams alloc] init];
    
    params.playlistUID = selectedPlaylist.UID;
    params.name = newName;
    params.type = selectedPlaylist.type;
    
    [[PCAPISDK sharedInstance] editPlaylist:params withCompletionHandler:^(PCPlaylist *playlist, PCAPIQueryResponse *response) {
        
        [blocksafeSelf didFinishDoingAction:renamePlaylistAction onPlaylist:selectedPlaylist result:playlist response:response];
    }];
}

- (void)renamePlaylistActionCompletion:(PCPlaylist *)selectedPlaylist :(UIAlertController *)alertController {
    
    __weak UIViewController *blocksafeSelf = self;
    
    alertController = [UIAlertController alertControllerWithTitle:NSLocalizedString(@"Enter a new name", nil)
                                                          message:nil
                                                   preferredStyle:UIAlertControllerStyleAlert];
    
    [self alertController:alertController showTextInputAlertWithTitle:selectedPlaylist.name andOKCompletionHandler:^(UIAlertAction *action) {
        UITextField *textField = [alertController.textFields firstObject];
        if (textField.text.length) {
            
            [blocksafeSelf didBeginDoingPlaylistAction:renamePlaylistAction];
            [blocksafeSelf renamePlaylist:selectedPlaylist :textField.text];
        }
    }];
}

- (void)addRenameAction:(UIAlertController *)alertController forSelectedPlaylist:(PCPlaylist *)selectedPlaylist {
    __weak UIViewController *blocksafeSelf = self;
    __block UIAlertController *renamePlaylistAlertController;

    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Rename", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    
                    [blocksafeSelf renamePlaylistActionCompletion:selectedPlaylist :renamePlaylistAlertController];
                }];
}

#pragma mark -
#pragma mark Delete

- (void)deletePlaylist:(PCPlaylist *)playlist {
    __weak UIViewController *blocksafeSelf = self;

    PCDeletePlaylistAPIParams *params = [[PCDeletePlaylistAPIParams alloc] init];
    params.playlistUID = playlist.UID;
    [[PCAPISDK sharedInstance] deletePlaylist:params withCompletionHandler:^(PCAPIQueryResponse *response) {
        
        [blocksafeSelf didFinishDoingAction:deletePlaylistAction onPlaylist:playlist result:nil response:response];
    }];

}

- (void)deletePlaylistActionCompletion:(PCPlaylist *)playlist {
    __weak UIViewController *blocksafeSelf = self;
    
    NSString *message = NSLocalizedString(@"Do you want to delete this Playlist", nil);
    
    if (playlist.playlistType == playlistTypeImageVideo) {
        
        message = NSLocalizedString(@"Do you want to delete this Album", nil);
    }
    
    [self showOKCancelAlertWithTitle:NSLocalizedString(@"Warning", nil)
                          andMessage:message
                            okAction:^(UIAlertAction *action) {
                                [blocksafeSelf didBeginDoingPlaylistAction:deletePlaylistAction];
                                [blocksafeSelf deletePlaylist:playlist];
                            }
                        cancelAction:^(UIAlertAction *action) {

                        }];
}

- (void)addDeleteAction:(UIAlertController *)alertController forSelectedResource:(PCPlaylist *)selectedPlaylist {
    __weak UIViewController *blocksafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Delete", nil)
                    style:UIAlertActionStyleDestructive
                andAction:^(UIAlertAction * _Nonnull action) {

                    [blocksafeSelf deletePlaylistActionCompletion:selectedPlaylist];
                }];
}


#pragma mark -
#pragma mark Playlist actions

- (void)showActionsForSelectedPlaylist:(PCPlaylist *)selectedPlaylist {
    
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:selectedPlaylist.name
                                                                             message:nil
                                                                      preferredStyle:UIAlertControllerStyleActionSheet];
    
    
   
    [self addRenameAction:alertController forSelectedPlaylist:selectedPlaylist];
    [self addDeleteAction:alertController forSelectedResource:selectedPlaylist];
    [self addCancelAction:alertController];
    
    [self presentViewController:alertController animated:YES completion:nil];
}


@end
