//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import <UIKit/UIKit.h>

typedef enum {

#ifdef KEPLER_FILE_BROWSER
    copyResourceAction,
    moveResourceAction,
    renameResourceAction,
    deleteResourceAction,
    restoreResourceAction,
    
#endif
#ifdef HUBBLE_MEDIA_BROWSER
    fileInfoResourceAction,
    addToAlbumResourceAction,
    addToPlaylistResourceAction,
    removeFromPlaylistResourceAction,
    removeFromAlbumResourceAction,
#endif
    addToGalleryResourceAction
} ResourceAction;

@class PCResource;
@interface UIViewController (ResourceActions)

- (NSString *)stringForResourceAction:(ResourceAction)action;

/*!
 * call this method to trigger file/folder action's like delete,rename, move and copy
 * @param resource resource on which action needs to be performed
 */
- (void)resourceSelected:(PCResource *)resource showActions:(id) firstAction, ... NS_REQUIRES_NIL_TERMINATION;

/*!
 * Implement this action in classes which consume this category if they needs to pass any data inorder to perform a particular action. For eg a root path in Folder create operation
 */
- (id)dataSourceForResourceAction:(ResourceAction)action;

/*!
 * Implement this method in the viewController consuming this category class. This method can implement cusotm actions which needs to be implemented before performing a particular action. For eg. start a spinner
 * @param action the action that is going to be executed
 */
- (void)didBeginDoingResourceAction:(ResourceAction)action;

/*!
 * Implement this method in the viewController consuming this category class. This method can implement cusotm actions which needs to be implemented after performing a particular action. For eg. stop a spinner, reload a gridview etc
 * @param action the action that is being executed
 * @param result result of action, if any
 * @param response response of the action, if any
 */
- (void)didFinishDoingAction:(ResourceAction)action onResource:(id)resource result:(id)result response:(id)response;

@end
