//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import "UIViewController+ResourceActions.h"
#import "UIViewController+Alert.h"
#ifdef HUBBLE_MEDIA_BROWSER
#import "PCResource+Info.h"
#import "MoveToPlaylistViewController.h"
#endif
#ifdef KEPLER_FILE_BROWSER
#import "MetaDataViewController.h"
#import "MoveViewController.h"
#import "CopyViewController.h"
#import "PCResource+Info.h"
#import "NSString+Addition.h"
#endif


@implementation UIViewController (ResourceActions)


- (NSString *)stringForResourceAction:(ResourceAction)action {

    switch (action) {
#ifdef KEPLER_FILE_BROWSER

        case copyResourceAction:
            return NSLocalizedString(@"Copy", nil);
            break;
        case moveResourceAction:
            return NSLocalizedString(@"Move", nil);
            break;
        case renameResourceAction:
            return NSLocalizedString(@"Rename", nil);
            break;
        case deleteResourceAction:
            return NSLocalizedString(@"delete", nil);
        case restoreResourceAction:
            return NSLocalizedString(@"Restore", nil);
            break;
#endif
#ifdef HUBBLE_MEDIA_BROWSER
        case addToAlbumResourceAction:
            return NSLocalizedString(@"Add to Album", nil);
            break;
        case addToPlaylistResourceAction:
            return NSLocalizedString(@"Add to Playlist", nil);
            break;
        case removeFromAlbumResourceAction:
            return NSLocalizedString(@"Remove from Album", nil);
            break;
        case removeFromPlaylistResourceAction:
            return NSLocalizedString(@"Remove from Playlist", nil);
            break;
#endif
        case addToGalleryResourceAction:
            return NSLocalizedString(@"Add to gallery", nil);
            break;
        default:
            return nil;
            break;
    }
}

- (void)didBeginDoingResourceAction:(ResourceAction)action {
    // abstract method
}

- (void)didFinishDoingAction:(ResourceAction)action onResource:(id)resource result:(id)result response:(id)response {
    // abstract method
}

- (id)dataSourceForResourceAction:(ResourceAction)action {
    // abstract method
    return nil;
}

#pragma mark -
#pragma mark Cancel
- (void)addCancelAction:(UIAlertController *)alertController {
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Cancel", nil)
                    style:UIAlertActionStyleCancel
                andAction:nil];
}

#ifdef KEPLER_FILE_BROWSER

#pragma mark -
#pragma mark Rename

- (void)showRenameAlert:(UIAlertController *)alertController forSelectedResource:(PCResource *)selectedResource {
    
    __weak UIViewController *blocksafeSelf = self;
    
    alertController = [UIAlertController alertControllerWithTitle:NSLocalizedString(@"Enter a new name", nil)
                                                          message:nil
                                                   preferredStyle:UIAlertControllerStyleAlert];

    [self alertController:alertController showTextInputAlertWithTitle:selectedResource.name andOKCompletionHandler:^(UIAlertAction *action) {
        UITextField *textField = [alertController.textFields firstObject];
        if (textField.text.length) {
            
            [blocksafeSelf didBeginDoingResourceAction:renameResourceAction];
            
            PCFOPSAPIParams *rename = [[PCFOPSAPIParams alloc] init];
            
            rename.source = selectedResource.fullPath;
            rename.target = [NSString stringWithFormat:@"%@/%@", selectedResource.parentPath, textField.text];
            
            
            [[PCAPISDK sharedInstance] rename:rename withCompletionHandler:^(PCResource *metadata, PCAPIQueryResponse *response) {
                
                [blocksafeSelf didFinishDoingAction:renameResourceAction onResource:selectedResource result:metadata response:response];
            }];
        }
    }];
}

- (void)addRenameAction:(UIAlertController *)alertController forSelectedResource:(PCResource *)selectedResource {
    __weak UIViewController *blocksafeSelf = self;
    
    __block UIAlertController *renameAlertController = nil;
    
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Rename", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    
                    [blocksafeSelf showRenameAlert:renameAlertController forSelectedResource:selectedResource];
                }];
}
#pragma mark -
#pragma mark Restore

- (void)addRestoreAction:(UIAlertController *)alertController forSelectedResource:(PCResource *)selectedResource {
    __weak UIViewController *blocksafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Restore", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    
                    NSString *message = NSLocalizedString(@"Are you sure want to restore folder", nil);
                    
                    if (selectedResource.fileType == imageFileType || selectedResource.fileType == musicFileType ||  selectedResource.fileType == videoFileType) {
                        
                        message = NSLocalizedString(@"Are you sure want to restore file", nil);
                        
                    }
                    
                    [blocksafeSelf showOKCancelAlertWithTitle:NSLocalizedString(@"Alert", nil)
                                          andMessage:message
                                            okAction:^(UIAlertAction *action) {
                                                
                                            PCRestoreAPIParams *restoreParams = [[PCRestoreAPIParams alloc] init];
                                                
                                            restoreParams.paths = @[selectedResource.fullPath];
                                        
                                            [blocksafeSelf didBeginDoingResourceAction:restoreResourceAction];
                                            [[PCAPISDK sharedInstance] restore:restoreParams withCompletionHandler:^(PCAPIQueryResponse *response) {
                                                        [blocksafeSelf didFinishDoingAction:restoreResourceAction onResource:selectedResource result:nil response:response];
                                            }];
                                            }
                                        cancelAction:^(UIAlertAction *action) {
                                            
                                        }];
                    
    }];

}
#pragma mark -
#pragma mark Delete

- (void)addDeleteAction:(UIAlertController *)alertController forSelectedResource:(PCResource *)selectedResource {
    __weak UIViewController *blocksafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Delete", nil)
                    style:UIAlertActionStyleDestructive
                andAction:^(UIAlertAction * _Nonnull action) {
            
                    NSString *message = NSLocalizedString(@"Are you sure want to delete folder", nil);
                    
                    if (selectedResource.fileType == imageFileType || selectedResource.fileType == musicFileType ||  selectedResource.fileType == videoFileType) {
                        
                        message = NSLocalizedString(@"Are you sure want to delete file", nil);
                        
                    }
                    
                    [blocksafeSelf showOKCancelAlertWithTitle:NSLocalizedString(@"Warning", nil)
                                          andMessage:message
                                            okAction:^(UIAlertAction *action) {
                                                
                                                PCDeleteParams *params = [[PCDeleteParams alloc] init];
                                                params.path = selectedResource.fullPath;
                                                [blocksafeSelf didBeginDoingResourceAction:deleteResourceAction];
                                                [[PCAPISDK sharedInstance] deletePath:params withCompletionHandler:^(PCAPIQueryResponse *response) {
                                                    
                                                    [blocksafeSelf didFinishDoingAction:deleteResourceAction onResource:selectedResource result:nil response:response];
                                                }];
                                                
                                            }
                                        cancelAction:^(UIAlertAction *action) {
                                            
                                        }];
                }];
}

#pragma mark -
#pragma Move

- (void)addMoveAction:(UIAlertController *)alertController forSelectedResource:(PCResource *)selectedResource {
    __weak UIViewController *blocksafeSelf = self;
    
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Move", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    
                    UIStoryboard *storyBoard = [UIStoryboard storyboardWithName:@"Main" bundle:nil];
                    MoveViewController *controller = (MoveViewController*)[storyBoard instantiateViewControllerWithIdentifier: @"MoveViewController"];
                    controller.source = selectedResource.fullPath;
                    [controller setCompletionHandler:^(PCResource *metadata, PCAPIQueryResponse *response) {

                        [blocksafeSelf didFinishDoingAction:moveResourceAction onResource:selectedResource result:metadata response:response];
                    }];
                   
                    UINavigationController *navController = [[UINavigationController alloc] initWithRootViewController:controller];
                    [blocksafeSelf presentViewController:navController animated:YES completion:nil];
                }];
}

#pragma mark -
#pragma mark Copy

- (void)addCopyAction:(UIAlertController *)alertController forSelectedResource:(PCResource *)selectedResource {
    __weak UIViewController *blocksafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Copy", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    
                    UIStoryboard *storyBoard = [UIStoryboard storyboardWithName:@"Main" bundle:nil];
                    CopyViewController *controller = (CopyViewController*)[storyBoard instantiateViewControllerWithIdentifier: @"CopyViewController"];
                    controller.source = selectedResource.fullPath;
                    [controller setCompletionHandler:^(PCResource *metadata, PCAPIQueryResponse *response){
                        
                        [blocksafeSelf didFinishDoingAction:copyResourceAction onResource:selectedResource result:metadata response:response];
                    }];
                    
                    UINavigationController *navController = [[UINavigationController alloc] initWithRootViewController:controller];
                    [blocksafeSelf presentViewController:navController animated:YES completion:nil];
                }];
}

#endif

#ifdef HUBBLE_MEDIA_BROWSER

- (void)addFileInfoAction:(UIAlertController *)alertController forSelectedResource:(PCResource *)selectedResource {
    __weak UIViewController *blocksafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"File Info", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    
                    [blocksafeSelf showOKAlertWithTitle:NSLocalizedString(@"File Info", nil)
                                    andMessage:nil
                              attributedString:[selectedResource fileInfo]
                                      okAction:nil];

                }];
}

- (void)addToPlaylistAction:(ResourceAction)resourceAction controller:(UIAlertController *)alertController forSelectedResource:(PCResource *)selectedResource {

    
    
    __weak UIViewController *blocksafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:[self stringForResourceAction:resourceAction]
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    
                    
                    UIStoryboard *storyBoard = [UIStoryboard storyboardWithName:@"Main" bundle:nil];
                    MoveToPlaylistViewController *controller = (MoveToPlaylistViewController *)[storyBoard instantiateViewControllerWithIdentifier:@"MoveToPlaylistViewController"];
                    controller.selectedResource = selectedResource;
                    [controller setCompletionHandler:^(NSArray *playlistItems, PCAPIQueryResponse *response){
                        
                        [blocksafeSelf didFinishDoingAction:resourceAction onResource:selectedResource result:playlistItems response:response];
                    }];
                    
                    UINavigationController *navController = [[UINavigationController alloc] initWithRootViewController:controller];
                    [blocksafeSelf presentViewController:navController animated:YES completion:nil];
                }];

}

- (void)addRemoveFromPlaylistAction:(ResourceAction)resourceAction controller:(UIAlertController *)alertController forSelectedResource:(PCResource *)selectedResource {
    __weak UIViewController *blocksafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:[self stringForResourceAction:resourceAction]
                    style:UIAlertActionStyleDestructive
                andAction:^(UIAlertAction * _Nonnull action) {
                    
                    
                    PCDeletePlaylistItemAPIParams *params = [[PCDeletePlaylistItemAPIParams alloc] init];
                    params.playlistUID = [self dataSourceForResourceAction:resourceAction];
                    params.itemUID = selectedResource.itemUID;
                    [blocksafeSelf didBeginDoingResourceAction:resourceAction];
                    [[PCAPISDK sharedInstance] deletePlaylistItem:params withCompletionHandler:^(PCAPIQueryResponse *response) {
                        
                        [blocksafeSelf didFinishDoingAction:resourceAction onResource:selectedResource result:nil response:response];
                    }];
                }];
}



#endif

- (void)addToGalleryAction:(ResourceAction)resourceAction controller:(UIAlertController *)alertController forSelectedResource:(PCResource *)selectedResource {
    [self alertController:alertController
       addActionWithTitle:[self stringForResourceAction:resourceAction]
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    
                    UIImage *image = [self dataSourceForResourceAction:resourceAction];
                    UIImageWriteToSavedPhotosAlbum(image, self, @selector(image:didFinishSavingWithError:contextInfo:), (__bridge void * _Nullable)(selectedResource));
                }];
}

- (void)image:(UIImage *)image didFinishSavingWithError:(NSError *)error contextInfo:(PCResource *)resource; {
    [self didFinishDoingAction:addToGalleryResourceAction onResource:resource result:nil response:error];
}


#pragma mark -
#pragma mark All action

- (void)resourceSelected:(PCResource *)selectedResource showActions:(id) firstAction, ... {
    
    // first collect the actions y iterating through them
    // duplicate actions are ignored, hence chosen a NSMutableSet
    NSMutableArray *arguments = [[NSMutableArray alloc] init];
    id eachObject;
    va_list argumentList;
    if (firstAction)  {
        [arguments addObject:firstAction];
        va_start(argumentList, firstAction);
        while ((eachObject = va_arg(argumentList, id)))  {
            [arguments addObject: eachObject];
        }
        va_end(argumentList);
    }
    
    
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:selectedResource.name
                                                                             message:nil
                                                                      preferredStyle:UIAlertControllerStyleActionSheet];
    
    
    
    for (NSNumber *actionObj in arguments) {
        switch ([actionObj intValue]) {
#ifdef KEPLER_FILE_BROWSER
                
            case copyResourceAction:
                [self addCopyAction:alertController forSelectedResource:selectedResource];
                break;
            case moveResourceAction:
                [self addMoveAction:alertController forSelectedResource:selectedResource];
                break;
            case deleteResourceAction:
                [self addDeleteAction:alertController forSelectedResource:selectedResource];
                break;
            case renameResourceAction:
                [self addRenameAction:alertController forSelectedResource:selectedResource];
                break;
            case restoreResourceAction:
                [self addRestoreAction:alertController forSelectedResource:selectedResource];
                break;
#endif
#ifdef HUBBLE_MEDIA_BROWSER
            case fileInfoResourceAction: {

                [self addFileInfoAction:alertController forSelectedResource:selectedResource];
            }
                break;
            case addToPlaylistResourceAction:
            case addToAlbumResourceAction:
                [self addToPlaylistAction:[actionObj intValue] controller:alertController forSelectedResource:selectedResource];
                break;
            case removeFromPlaylistResourceAction:
            case removeFromAlbumResourceAction:
                [self addRemoveFromPlaylistAction:[actionObj intValue] controller:alertController forSelectedResource:selectedResource];
                break;
#endif
            case addToGalleryResourceAction:
                [self addToGalleryAction:[actionObj intValue] controller:alertController forSelectedResource:selectedResource];
                break;
            default:
                break;
        }
    }
    
    [self addCancelAction:alertController];
    [self presentViewController:alertController animated:YES completion:nil];
}

@end
