//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import "UIViewController+RightBarButtonAction.h"
#import "SearchConfigurationViewController.h"
#import "UIViewController+Alert.h"
#import "PCAccountInfo+additions.h"
#import "UIViewController+Dismiss.h"
#import "UIViewController+Alert.h"
#ifdef KEPLER_FILE_BROWSER
#import <MobileCoreServices/MobileCoreServices.h>
#import <AssetsLibrary/AssetsLibrary.h>
#import "UploadManager.h"
#endif

@interface UIViewController () <UIImagePickerControllerDelegate, UINavigationControllerDelegate>

@end

@interface ActionBarButtonItem : UIBarButtonItem

@property (nonatomic, strong) NSArray *actions;

@end

@implementation ActionBarButtonItem


@end


@implementation UIViewController (RightBarButtonAction)

- (NSString *)stringForRightBarAction:(rightBarAction)action {

    switch (action) {
        case accountInfoRightBarAction:
            return NSLocalizedString(@"Account Info", nil);
            break;
        case searchRightBarAction:
            return NSLocalizedString(@"Search", nil);
            break;
#ifdef KEPLER_FILE_BROWSER
        case emptyTrashRightBarAction:
            return NSLocalizedString(@"Empty Trash", nil);
            break;
        case createFolderRightBarAction:
            return NSLocalizedString(@"Create Folder", nil);
            break;
        case uploadFileRightBarAction:
            return NSLocalizedString(@"Upload File", nil);
            break;
#endif
#ifdef HUBBLE_MEDIA_BROWSER
        case createPlaylistRightBarAction:
            return NSLocalizedString(@"Create Playlist", nil);
            break;
        case createAlbumRightBarAction:
            return NSLocalizedString(@"Create Album", nil);
            break;
#endif
    case signOutRightBarAction:
            
            return NSLocalizedString(@"Sign Out", nil);
            break;
    case refreshAccountBarAction:
            return NSLocalizedString(@"refresh account", nil);
            break;
        default:
            return nil;
            break;
    }
}

- (void)didBeginDoingRightBarButtonAction:(rightBarAction)action {
    // abstract method, supposed to be executed by subclasses
}

- (void)didFinishDoingRightBarButtonAction:(rightBarAction)action :(id)result :(id)response {
    // abstract method, supposed to be executed by subclasses
}

- (id)dataSourceForRightBarButtonAction:(rightBarAction)action {
    return nil;
}

- (void)addRightBarButtonItemWithActions:(id) firstAction, ... {
    
    // first collect the actions y iterating through them
    // duplicate actions are ignored, hence chosen a NSMutableSet
    NSMutableArray *arguments = [[NSMutableArray alloc] init];
    id eachObject;
    va_list argumentList;
    if (firstAction)  {
        [arguments addObject:firstAction];
        va_start(argumentList, firstAction);
        while ((eachObject = va_arg(argumentList, id)))  {
            [arguments addObject: eachObject];
        }
        va_end(argumentList);
    }

    
    // create the bar button item using the actions
    ActionBarButtonItem *actionButton = [[ActionBarButtonItem alloc]
                                     initWithBarButtonSystemItem:UIBarButtonSystemItemAction
                                     target:self
                                     action:@selector(rightBarButtonClicked:)];
    actionButton.actions = arguments;
    
    self.navigationItem.rightBarButtonItem = actionButton;
}

- (void)executeAction:(rightBarAction)action silent:(BOOL)silent {
    switch (action) {
#ifdef KEPLER_FILE_BROWSER
        case emptyTrashRightBarAction:
            [self addEmptyTrashAction:nil];
            break;
            
        case createFolderRightBarAction:
            [self doCreateFolderActionCompletion:nil];
            break;
#endif
        case accountInfoRightBarAction:
            [self doAccountInfoActionCompletion];
            break;
        case searchRightBarAction:
            [self doSearchActionCompletion];
            break;
        case signOutRightBarAction:
            [self doLogoutActionCompletion:silent];
            break;
        case refreshAccountBarAction:
            [self doRefreshAccountCompletion];
            break;
#ifdef HUBBLE_MEDIA_BROWSER
        case createPlaylistRightBarAction:
        case createAlbumRightBarAction:
            [self doCreatePlaylistAction:action completion:nil];
            break;
#endif
        default:
            break;
    }
}


#pragma mark -
- (void)rightBarButtonClicked:(id)sender {
    
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:nil
                                                                             message:nil
                                                                      preferredStyle:UIAlertControllerStyleActionSheet];

    ActionBarButtonItem *senderButton = (ActionBarButtonItem *)sender;
    
    for (NSNumber *actionObj in senderButton.actions) {
        switch ([actionObj intValue]) {
            case accountInfoRightBarAction:
                [self addAccountInfoAction:alertController];
                break;
            case searchRightBarAction:
                [self addSearchAction:alertController];
                break;
            case signOutRightBarAction:
                [self addSignOutAction:alertController];
                break;
            case refreshAccountBarAction:
                [self addRefreshAccountAction:alertController];
                break;
#ifdef KEPLER_FILE_BROWSER
            case emptyTrashRightBarAction:
                [self addEmptyTrashAction:alertController];
                break;
            case createFolderRightBarAction:
                [self addCreateFolderAction:alertController];
                break;
            case uploadFileRightBarAction:
                [self addUploadFileAction:alertController];
                break;
#endif
#ifdef HUBBLE_MEDIA_BROWSER
            case createPlaylistRightBarAction:
            case createAlbumRightBarAction:
                [self addCreatePlaylistAction:[actionObj intValue] alertController:alertController];
                break;
#endif
            default:
                break;
        }
    }

    [self addCancelAction:alertController];
    [self presentViewController:alertController animated:YES completion:nil];
    
}


#pragma mark -
#pragma mark Cancel

- (void)addCancelAction:(UIAlertController *)alertController {
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Cancel", nil)
                    style:UIAlertActionStyleCancel
                andAction:nil];
}

#ifdef KEPLER_FILE_BROWSER
#pragma mark -
#pragma mark Create Folder

- (void)doCreateFolderActionCompletion:(UIAlertController *)alertController {
    __weak UIViewController *blocksafeSelf = self;

    alertController = [UIAlertController alertControllerWithTitle:NSLocalizedString(@"Enter Folder name", nil)
                                                          message:nil
                                                   preferredStyle:UIAlertControllerStyleAlert];

    [self alertController:alertController showTextInputAlertWithTitle:nil andOKCompletionHandler:^(UIAlertAction *action) {
        UITextField *textField = [alertController.textFields firstObject];
        if (textField.text.length) {
            [blocksafeSelf didBeginDoingRightBarButtonAction:createFolderRightBarAction];
            PCCreateFolderAPIParams *createFolderParams = [[PCCreateFolderAPIParams alloc] init];
            createFolderParams.name = textField.text;
            createFolderParams.path = [blocksafeSelf dataSourceForRightBarButtonAction:createFolderRightBarAction];
            [[PCAPISDK sharedInstance] createFolder:createFolderParams withCompletionHandler:^(PCResource *metadata, PCAPIQueryResponse *response) {
                
                [blocksafeSelf didFinishDoingRightBarButtonAction:createFolderRightBarAction :metadata :response];
            }];
        }
    }];
}

- (void)addCreateFolderAction:(UIAlertController *)alertController {
    __weak UIViewController *blocksafeSelf = self;
    __block UIAlertController *createFolderAlertController;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Create Folder", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    [blocksafeSelf doCreateFolderActionCompletion:createFolderAlertController];
                }];
}

#pragma mark -
#pragma mark Empty Trash
- (void)addEmptyTrashAction:(UIAlertController *)alertController {
    __weak UIViewController *blocksafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Empty Trash", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    [blocksafeSelf emptyTrashAction];
                }];
}

- (void)emptyTrashAction {
    __weak UIViewController *blocksafeSelf = self;
    [self showOKCancelAlertWithTitle:NSLocalizedString(@"Empty Trash", nil) andMessage:NSLocalizedString(@"Do you wish to empty trash?", nil) okAction:^(UIAlertAction *action) {
        [blocksafeSelf didBeginDoingRightBarButtonAction:emptyTrashRightBarAction];
        [[PCAPISDK sharedInstance] deleteTrashWithCompletionHandler:^(PCAPIQueryResponse *response) {
            [blocksafeSelf didFinishDoingRightBarButtonAction:emptyTrashRightBarAction :nil :response];
        }];
        
    } cancelAction:nil];
}

#pragma mark -
#pragma mark Upload File


- (void)imagePickerController:(UIImagePickerController *)picker didFinishPickingMediaWithInfo:(NSDictionary *)info {
    
    [picker dismissViewControllerAnimated:YES completion:NULL];
    
    __block NSString *filePath = nil;
    
    
    NSURL *assetURL = [info valueForKey:UIImagePickerControllerReferenceURL];
    
    ALAssetsLibraryAssetForURLResultBlock resultblock = ^(ALAsset *assetItem) {
        
        NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
        NSString *documentsDirectory = [paths objectAtIndex:0];
        
        ALAssetRepresentation *representation = [assetItem defaultRepresentation];
        NSString *fileName = [representation filename];
        filePath = [documentsDirectory stringByAppendingPathComponent:fileName];
        
        
        if (info[UIImagePickerControllerMediaType] == (NSString*)kUTTypeImage) {
            
            NSData *imageData = UIImagePNGRepresentation(info[UIImagePickerControllerOriginalImage]);
            [imageData writeToFile:filePath atomically:YES];
            
        } else if (info[UIImagePickerControllerMediaType] == (NSString*)kUTTypeMovie) {
            
            NSURL *videoURL = info[UIImagePickerControllerMediaURL];
            filePath = [videoURL path];
        }
        
        NSString *folderPath = [self dataSourceForRightBarButtonAction:createFolderRightBarAction];
        
        UploadObj *obj = [[UploadObj alloc] init];
        obj.fileName = fileName;
        obj.destinationFolder = folderPath;
        obj.filePath = filePath;
        obj.thumbnailImage = [UIImage imageWithCGImage:[assetItem thumbnail]];
        
        [[UploadManager sharedInstance] addFileToUploadQueue:obj];

    };
    
    ALAssetsLibraryAccessFailureBlock failureblock  = ^(NSError *myerror) {
        NSLog(@"Can not get asset - %@",[myerror localizedDescription]);
    };
    
    
    if (assetURL.absoluteString.length) {
        ALAssetsLibrary *assetsLibrary = [[ALAssetsLibrary alloc] init];
        [assetsLibrary assetForURL:assetURL
                       resultBlock:resultblock
                      failureBlock:failureblock];
    }
}

- (void)imagePickerControllerDidCancel:(UIImagePickerController *)picker {
    [picker dismissViewControllerAnimated:YES completion:NULL];
}



- (void)doUploadFileActionCompletion {
    UIImagePickerController *picker = [[UIImagePickerController alloc] init];
    picker.delegate = self;
    picker.sourceType = UIImagePickerControllerSourceTypePhotoLibrary;
    picker.mediaTypes = [[NSArray alloc] initWithObjects: (NSString *) kUTTypeMovie, (NSString*) kUTTypeImage, nil];
    [self presentViewController:picker animated:YES completion:NULL];
}

- (void)addUploadFileAction:(UIAlertController *)alertController {
    __weak UIViewController *blockSafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Upload File", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    [blockSafeSelf doUploadFileActionCompletion];
                }];
}


#endif

#pragma mark -
#pragma mark Search

- (void)doSearchActionCompletion {

    UIStoryboard *storyBoard = [UIStoryboard storyboardWithName:@"Main" bundle:nil];
    SearchConfigurationViewController *controller = (SearchConfigurationViewController*)[storyBoard instantiateViewControllerWithIdentifier: @"SearchConfigurationViewController"];
#ifdef HUBBLE_MEDIA_BROWSER
    controller.hidesBottomBarWhenPushed = YES;
#endif
    [self.navigationController pushViewController:controller animated:YES];

}

- (void)addSearchAction:(UIAlertController *)alertController {
    __weak UIViewController *blockSafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Search", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {

                    [blockSafeSelf doSearchActionCompletion];
                }];
}

#pragma mark -
#pragma mark AccountInfo

- (void)fetchAccountInfo {
    __weak UIViewController *blockSafeSelf = self;

    [[PCAPISDK sharedInstance] fetchAccountInfoWithCompletionHandler:^(PCAccountInfo *accountInfo, PCAPIQueryResponse *response) {
        
        [blockSafeSelf didFinishDoingRightBarButtonAction:accountInfoRightBarAction :accountInfo :response];
        
    }];
}

- (void)doAccountInfoActionCompletion {
    [self didBeginDoingRightBarButtonAction:accountInfoRightBarAction];
    [self fetchAccountInfo];
}

- (void)addAccountInfoAction:(UIAlertController *)alertController {
    __weak UIViewController *blockSafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Account Info", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    [blockSafeSelf doAccountInfoActionCompletion];
                }];
}

#pragma mark -
#pragma mark Refresh Account

- (void)refreshAccount {
    __weak UIViewController *blocksafeSelf = self;
    [[PCAPISDK sharedInstance] refreshSessionWithCompletionHandler:^(PCAuthenticationResponse *authenticationCredentials, PCAPIQueryResponse *response) {
        [blocksafeSelf didFinishDoingRightBarButtonAction:refreshAccountBarAction :authenticationCredentials :response];
    }];
}

- (void)doRefreshAccountCompletion {

    [self didBeginDoingRightBarButtonAction:refreshAccountBarAction];
    [self refreshAccount];
}

- (void)addRefreshAccountAction:(UIAlertController *)alertController {
    __weak UIViewController *blocksafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Refresh Account", nil)
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    [blocksafeSelf doRefreshAccountCompletion];
                }];
}

#pragma mark -
#pragma mark SignOut

- (void)logOutUser {
    
    [[PCAPISDK sharedInstance] logout];
#ifdef KEPLER_FILE_BROWSER
    [[UploadManager sharedInstance] cleanUpload];
#endif
    [self dismissSelfRecursivelyAnimated:YES];
}

- (void)doLogoutActionCompletion:(BOOL)silent {
   
    __weak UIViewController *blocksafeSelf = self;
    if (silent) {
        if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
            [blocksafeSelf logOutUser];
        }
    } else {
        [self showOKCancelAlertWithTitle:NSLocalizedString(@"Sign Out", nil) andMessage:NSLocalizedString(@"Do you wish to continue Sign Out?", nil) okAction:^(UIAlertAction *action) {
            if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
                [blocksafeSelf logOutUser];
            }
        } cancelAction:nil];
    }
}


- (void)addSignOutAction:(UIAlertController *)alertController {
    __weak UIViewController *blocksafeSelf = self;
    [self alertController:alertController
       addActionWithTitle:NSLocalizedString(@"Sign Out", nil)
                    style:UIAlertActionStyleDestructive
                andAction:^(UIAlertAction * _Nonnull action) {
                    [blocksafeSelf doLogoutActionCompletion:NO];
                }];
}

#ifdef HUBBLE_MEDIA_BROWSER
#pragma mark -
#pragma mark Playlist

- (void)doCreatePlaylistAction:(rightBarAction)buttonAction completion:(UIAlertController *)alertController {

    __weak UIViewController *blocksafeSelf = self;
 
    
    alertController = [UIAlertController alertControllerWithTitle:[self stringForRightBarAction:buttonAction]
                                                          message:nil
                                                   preferredStyle:UIAlertControllerStyleAlert];
    
    [self alertController:alertController showTextInputAlertWithTitle:nil andOKCompletionHandler:^(UIAlertAction *action) {
        UITextField *textField = [alertController.textFields firstObject];
        if (textField.text.length) {
            [blocksafeSelf didBeginDoingRightBarButtonAction:createPlaylistRightBarAction];
            PCCreateplaylistAPIParams *createPlaylistAPIParams = [[PCCreateplaylistAPIParams alloc] init];
            createPlaylistAPIParams.name = textField.text;
            createPlaylistAPIParams.type = [blocksafeSelf dataSourceForRightBarButtonAction:createPlaylistRightBarAction];
            [[PCAPISDK sharedInstance] createPlaylist:createPlaylistAPIParams withCompletionHandler:^(PCPlaylist *playlist, PCAPIQueryResponse *response) {
                [blocksafeSelf didFinishDoingRightBarButtonAction:createPlaylistRightBarAction :playlist :response];
            }];
        }
    }];
}

- (void)addCreatePlaylistAction:(rightBarAction)buttonAction alertController:(UIAlertController *)alertController {
    __weak UIViewController *blocksafeSelf = self;
    
    __block UIAlertController *createlaylistAlertController = nil;

    [self alertController:alertController
       addActionWithTitle:[self stringForRightBarAction:buttonAction]
                    style:UIAlertActionStyleDefault
                andAction:^(UIAlertAction * _Nonnull action) {
                    [blocksafeSelf doCreatePlaylistAction:buttonAction completion:createlaylistAlertController];
                }];
}

#endif


@end
