//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import <UIKit/UIKit.h>

@class PCResource, PCAPIQueryResponse, UploadInfo;

@interface UploadObj : NSObject

@property (nonatomic, strong) UIImage *thumbnailImage;
@property (nonatomic, strong) NSString *fileName;
@property (nonatomic, strong) NSString *filePath;
@property (nonatomic, strong) NSString *destinationFolder;
@property (nonatomic, strong) UploadInfo *uploadInfo;

@end

@protocol UploadDelegate <NSObject>

- (void)uploadManagerDidAddFileToUploadQueue:(UploadObj *)file;
- (void)uploadManagerDidBeginUploadingFile:(UploadObj *)file;
- (void)uploadManagerDidProgressedUploadingFile:(UploadObj *)file progress:(CGFloat)progress state:(PCUploadAPIState)state;
- (void)uploadManagerDidFinishUploadingFile:(UploadObj *)file result:(PCResource *)metadata response:(PCAPIQueryResponse *)response;

@end

/*! uploadmanager upload files serially. It also informs the upload progress of individual files through it's delegate methods
 *
 */
@interface UploadManager : NSObject

@property (nonatomic, readonly, assign) BOOL isUploadInProgress; // indicates if upload processis in progress, set to NO once all upload is done.
@property (nonatomic, readonly, assign) int totalFilesDone; // total files those were ether uploaded or failed
@property (nonatomic, readonly, assign) int totalFilesToBeUploaded; // total number of files queued to uploaded
@property (nonatomic, readonly, assign) float progress; // upload progress of current file
@property (nonatomic, weak) id<UploadDelegate> delegate;
@property (nonatomic, readonly) UploadObj *uploadingFile;


+ (UploadManager *)sharedInstance;

- (void)addFileToUploadQueue:(UploadObj *)file;

- (void)cleanUpload;

@end
