//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import "MetaDataViewController.h"
#import "UIViewController+Alert.h"
#import "UIViewController+RightBarButtonAction.h"
#import "UIViewController+ResourceActions.h"
#import "UIViewController+Spinner.h"
#import "NSArray+Addition.h"
#import "UploadManager.h"
#import "ProgressView.h"

@interface FileGridViewController (Protected)

@property (nonatomic, strong) NSString *emptyViewMessage;
@property (nonatomic, retain) UIRefreshControl *refreshControl;

- (void)refreshView;
- (void)alertController:(UIAlertController *)alertController
     addActionWithTitle:(NSString *)title
                  style:(UIAlertActionStyle)style
              andAction:(void (^)(UIAlertAction * _Nonnull action))handler;

@end


@interface MetaDataViewController () <UploadDelegate>

@property (weak, nonatomic) IBOutlet ProgressView *progressView;

@end

@implementation MetaDataViewController

- (void)reloadData {
    if (self.rootResource) {
        self.title = self.rootResource.name;
    } else {
        self.title = NSLocalizedString(@"ROOT", nil);
    }
    
    
    PCMetaDataAPIParams *params = [[PCMetaDataAPIParams alloc] init];
    params.path = self.rootResource.fullPath;
    
    __weak MetaDataViewController *blocksafeSelf = self;
    
    [[PCAPISDK sharedInstance] fetchMetadata:params withCompletionHandler:^(PCMetadata *metadata, PCAPIQueryResponse *response) {
        [blocksafeSelf hideSpinner];
        [blocksafeSelf.refreshControl endRefreshing];

        switch (response.urlResponse.statusCode) {
            case PCAPIResponseCodeOK: {
                
                blocksafeSelf.dataSource = [metadata.subFolders arrayByAddingObjectsFromArray:metadata.subFiles];
                [blocksafeSelf refreshView];
            }
                break;
            case PCAPIResponseCodeUnauthorized:
                [self showUnAuthorizedAlert];
                break;
            case PCAPIResponseCodeServiceUnavailable:
                [self showServerUnavailableAlert];
                break;
            default:
                [self showOperationFailedAlert:NSLocalizedString(@"Metadata fetch", nil)];
                break;
        }       
    }];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    
    if (self.rootResource) {
        
        self.title = self.rootResource.name;
    } else {
        
        self.title = NSLocalizedString(@"ROOT", nil);
    }
    
    self.emptyViewMessage = NSLocalizedString(@"Folder is empty", nil);
    
    [self showSpinner];
    [self.refreshControl beginRefreshing];

    [self reloadData];
    

}

- (void)viewWillAppear:(BOOL)animated {

    [super viewWillAppear:animated];
    
    [UploadManager sharedInstance].delegate = self;
    
    if ([UploadManager sharedInstance].isUploadInProgress) {
        
        self.progressView.hidden = NO;
        self.progressView.progressBar.progress = [UploadManager sharedInstance].progress;
        self.progressView.subLabel.text = [NSString stringWithFormat:@"(%d/%d)", [UploadManager sharedInstance].totalFilesDone, [UploadManager sharedInstance].totalFilesToBeUploaded];
        self.progressView.imageView.image = [UploadManager sharedInstance].uploadingFile.thumbnailImage;
        self.progressView.mainLabel.text = [NSString stringWithFormat:NSLocalizedString(@"Uploading file %@", nil), [UploadManager sharedInstance].uploadingFile.fileName];


    } else {
        self.progressView.hidden = YES;
    }
}

#pragma mark -
#pragma Upload delegate

- (void)uploadManagerDidAddFileToUploadQueue:(UploadObj *)file {

    self.progressView.hidden = NO;
    self.progressView.subLabel.text = [NSString stringWithFormat:@"(%d/%d)", [UploadManager sharedInstance].totalFilesDone, [UploadManager sharedInstance].totalFilesToBeUploaded];
}

- (void)uploadManagerDidBeginUploadingFile:(UploadObj *)file {

    self.progressView.mainLabel.text = [NSString stringWithFormat:NSLocalizedString(@"Uploading file %@", nil), file.fileName];
    self.progressView.progressBar.progress = 0.0;
    self.progressView.imageView.image = file.thumbnailImage;
}

- (void)uploadManagerDidProgressedUploadingFile:(UploadObj *)file progress:(CGFloat)progress state:(PCUploadAPIState)state {
   
    switch (state) {
        case PCUploadAPIStateInitiatingUpload:
            self.progressView.mainLabel.text = [NSString stringWithFormat:NSLocalizedString(@"Initiating upload for file %@", nil), file.fileName];
            break;
        case PCUploadAPIStateCommitingUpload:
            self.progressView.mainLabel.text = [NSString stringWithFormat:NSLocalizedString(@"Finalizing upload for file %@", nil), file.fileName];
            break;
        default:
            self.progressView.mainLabel.text = [NSString stringWithFormat:NSLocalizedString(@"Uploading file %@", nil), file.fileName];
            break;
    }
    
    self.progressView.progressBar.progress = progress;
}

- (void)uploadManagerDidFinishUploadingFile:(UploadObj *)file result:(PCResource *)metadata response:(PCAPIQueryResponse *)response {

    
    if ([metadata.parentPath isEqualToString:self.rootResource.fullPath]) {
        
        [self didFinishDoingRightBarButtonAction:uploadFileRightBarAction :metadata :response];
    }
    

    self.progressView.progressBar.progress = 0;
    self.progressView.mainLabel.text = nil;
    self.progressView.imageView.image = nil;

    if ([UploadManager sharedInstance].isUploadInProgress) {
        
        self.progressView.subLabel.text = [NSString stringWithFormat:@"(%d/%d)", [UploadManager sharedInstance].totalFilesDone, [UploadManager sharedInstance].totalFilesToBeUploaded];
    } else {
    
        self.progressView.hidden = YES;
    }
}


- (id)dataSourceForRightBarButtonAction:(rightBarAction)action {
    id dataSource = [super dataSourceForRightBarButtonAction:action];
    switch (action) {
        case uploadFileRightBarAction:
        case createFolderRightBarAction: {
            return self.rootResource.fullPath;
        }
            break;
        default:
            return dataSource;
            break;
    }
}

@end
