//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import "TrashViewController.h"
#import "UIViewController+Spinner.h"
#import "UIViewController+Alert.h"
#import "UIViewController+RightBarButtonAction.h"


@interface FileGridViewController (Protected)

@property (nonatomic, strong) NSString *emptyViewMessage;
@property (nonatomic, retain) UIRefreshControl *refreshControl;

- (void)refreshView;
- (void)alertController:(UIAlertController *)alertController
     addActionWithTitle:(NSString *)title
                  style:(UIAlertActionStyle)style
              andAction:(void (^)(UIAlertAction * _Nonnull action))handler;

@end
@interface TrashViewController ()

@end

@implementation TrashViewController

- (void)reloadData {
    
    PCGetTrashAPIParams *params = [[PCGetTrashAPIParams alloc] init];
    
    __weak TrashViewController *blocksafeSelf = self;
    
    [[PCAPISDK sharedInstance] fetchTrash:params withCompletionHandler:^(NSArray *trashItems, PCAPIQueryResponse *response) {
                
        [blocksafeSelf hideSpinner];
        [blocksafeSelf.refreshControl endRefreshing];
        
        switch (response.urlResponse.statusCode) {
            case PCAPIResponseCodeOK: {
                blocksafeSelf.dataSource = [NSArray arrayWithArray:trashItems];
                [blocksafeSelf refreshView];
            }
                break;
            case PCAPIResponseCodeUnauthorized:
                [self showUnAuthorizedAlert];
                break;
            case PCAPIResponseCodeServiceUnavailable:
                [self showServerUnavailableAlert];
                break;
            default:
                [self showOperationFailedAlert:NSLocalizedString(@"Trash fetch", nil)];
                break;
        }
    }];
}

- (void)viewDidLoad {

    [super viewDidLoad];

    if (self.rootResource) {
        self.title = self.rootResource.name;
    } else {
        self.title = NSLocalizedString(@"Trash", nil);
    }
    
    self.emptyViewMessage = NSLocalizedString(@"No items found in trash", nil);
    
    [self showSpinner];
    [self.refreshControl beginRefreshing];
    
    [self reloadData];
    // Do any additional setup after loading the view.
}

- (id)dataSourceForRightBarButtonAction:(rightBarAction)action {
    id dataSource = [super dataSourceForRightBarButtonAction:action];
    switch (action) {
        case emptyTrashRightBarAction: {
            return @"VZMOBILE";
        }
            break;
        default:
            return dataSource;
            break;
    }
}

#pragma mark -
#pragma mark RightBarButtonAction callback

- (void)didBeginDoingRightBarButtonAction:(rightBarAction)action {
    
    [self showSpinner];
}

- (void)didFinishDoingRightBarButtonAction:(rightBarAction)action :(id)result :(id)response {
    
    [self hideSpinner];
    
    PCAPIQueryResponse *APIResponse = (PCAPIQueryResponse *)response;
    
    switch (APIResponse.urlResponse.statusCode) {
        case PCAPIResponseCodeOK: {
            
            switch (action) {
                case emptyTrashRightBarAction: {
                    [self showOKAlertWithTitle:NSLocalizedString(@"Success", nil) andMessage:NSLocalizedString(@"Trash empty successfully", nil) attributedString:nil okAction:nil];
                    
                    NSMutableArray *mutableDataSource = [NSMutableArray arrayWithArray:self.dataSource];
                    [mutableDataSource removeAllObjects];
                    self.dataSource = [NSArray arrayWithArray:mutableDataSource];
                    [self refreshView];
                }
                    break;
                default:
                    break;
            }
            
        }
            break;
        case PCAPIResponseCodeUnauthorized:
            [self showUnAuthorizedAlert];
            break;
        case PCAPIResponseCodeServiceUnavailable:
            [self showServerUnavailableAlert];
            break;
        default:
            [self showOperationFailedAlert:[self stringForRightBarAction:action]];
            break;
    }
}


- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
