//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import <XCTest/XCTest.h>
#import "PCTestConstants.h"

@interface PCMoveAPITests : XCTestCase

@end

@implementation PCMoveAPITests

- (void)setUp {
    [super setUp];
    // Put setup code here. This method is called before the invocation of each test method in the class.
}

- (void)tearDown {
    // Put teardown code here. This method is called after the invocation of each test method in the class.
    [super tearDown];
}


/*!
 * Test Case: auth token is a required parameter. SDK should throw an exception in case auth token is nil.
 */
- (void)testNilAuthToken {
    // If User is not logged in, copy should throw an exception
    if (![[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        PCFOPSAPIParams *params = [[PCFOPSAPIParams alloc] init];
        
        XCTAssertThrows([[PCAPISDK sharedInstance] move:params withCompletionHandler:^(PCResource *metadata, PCAPIQueryResponse *response) {}] ,@"move with nil access token");
    }
}

/*!
 * Test Case: source is a required parameter. SDK should throw an exception in case source is nil.
 */
- (void)testValidSourceForMoveAPI {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        PCFOPSAPIParams *params = [[PCFOPSAPIParams alloc] init];
        params.target = @"/VZMOBILE";
        XCTAssertThrows([[PCAPISDK sharedInstance] move:params withCompletionHandler:^(PCResource *metadata, PCAPIQueryResponse *response) {}] ,@"move without valid source");
    } else {
        XCTFail(@"Test Failed due to nil access token");
    }
}


/*!
 * Test Case: target is a required parameter. SDK should throw an exception in case target is nil.
 */
- (void) testValidTargetForMoveAPI {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        XCTestExpectation *expectation = [self expectationWithDescription:@"Test for valid target"];
        
        PCMetaDataAPIParams *params = [[PCMetaDataAPIParams alloc] init];
        params.path = @"/VZMOBILE";
        
        [[PCAPISDK sharedInstance] fetchMetadata:params withCompletionHandler:^(PCMetadata *metadata, PCAPIQueryResponse *response) {
            switch (response.urlResponse.statusCode) {
                case PCAPIResponseCodeOK: {
                    PCFOPSAPIParams *params = [[PCFOPSAPIParams alloc] init];
                    PCResource *resource = [metadata.subFiles objectAtIndex:0];
                    params.source = resource.fullPath;
                    XCTAssertThrows([[PCAPISDK sharedInstance] move:params withCompletionHandler:^(PCResource *metadata, PCAPIQueryResponse *response) {}] ,@"move without valid target");
                    [expectation fulfill];
                }
                    break;
                    
                default:
                    XCTFail(@"Test could not be completed because metadata API failed");
                    break;
            }
        }];
        [self waitForExpectationsWithTimeout:ExpectationTimeOut*2 handler:^(NSError *error) {
            
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
        }];
        
    } else {
        XCTFail(@"Test Failed due to nil access token");
    }
}


/*!
 * Test Case: Check when safe is set true and conflict resolve not set anyvalue , It should return 404 error.
 */

- (void) testWhenConflictResolveNotSetValue {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        XCTestExpectation *expectation = [self expectationWithDescription:@"Test when conflict resolve has invalid value"];
        
        PCMetaDataAPIParams *params = [[PCMetaDataAPIParams alloc] init];
        params.path = @"/VZMOBILE";
        
        [[PCAPISDK sharedInstance] fetchMetadata:params withCompletionHandler:^(PCMetadata *metadata, PCAPIQueryResponse *response) {
            switch (response.urlResponse.statusCode) {
                case PCAPIResponseCodeOK: {
                    PCFOPSAPIParams *params = [[PCFOPSAPIParams alloc] init];
                    PCResource *resource = [metadata.subFiles objectAtIndex:0];
                    params.source = resource.fullPath;
                    params.target = @"/VZMOBILE";
                    params.safe = @"true";
                    
                    [[PCAPISDK sharedInstance] move:params withCompletionHandler:^(PCResource *metadata, PCAPIQueryResponse *response) {
                        
                        switch (response.urlResponse.statusCode) {
                            case PCAPIResponseCodeConfict:
                                break;
                            default:
                                XCTFail(@"Test case failed. It should return 404 error");
                                break;
                        }
                        [expectation fulfill];
                    }];
                    
                }
                    break;
                    
                default:
                    XCTFail(@"Test could not be completed because metadata API failed");
                    break;
            }
        }];
        
        [self waitForExpectationsWithTimeout:ExpectationTimeOut*2 handler:^(NSError *error) {
            
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
        }];
        
    } else {
        XCTFail(@"Test Failed due to nil access token");
    }
}


/*!
 * Test Case: Check when safe is set true and conflict resolve set value as "copy" , It should copy file with new name.
 */

- (void) testWhenConflictResolveSetValue {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        XCTestExpectation *expectation = [self expectationWithDescription:@"Test when conflict resolve has invalid value"];
        
        PCMetaDataAPIParams *params = [[PCMetaDataAPIParams alloc] init];
        params.path = @"/VZMOBILE";
        
        [[PCAPISDK sharedInstance] fetchMetadata:params withCompletionHandler:^(PCMetadata *metadata1, PCAPIQueryResponse *response) {
            switch (response.urlResponse.statusCode) {
                case PCAPIResponseCodeOK: {
                    PCFOPSAPIParams *params = [[PCFOPSAPIParams alloc] init];
                    PCResource *resource = [metadata1.subFiles objectAtIndex:0];
                    params.source = resource.fullPath;
                    params.target = @"/VZMOBILE";
                    params.safe = @"true";
                    params.conflictsolve = @"copy";
                    
                    [[PCAPISDK sharedInstance] move:params withCompletionHandler:^(PCResource *metadata2, PCAPIQueryResponse *response) {
                        
                        switch (response.urlResponse.statusCode) {
                            case PCAPIResponseCodeOK:
                                if (metadata2.count < metadata1.count) {
                                    
                                    XCTFail(@"Test case failed.File not moved");
                                    
                                }
                                break;
                            default:
                                XCTFail(@"Test case failed. It should return 404 error");
                                break;
                        }
                        [expectation fulfill];
                    }];
                    
                }
                    break;
                    
                default:
                    XCTFail(@"Test could not be completed because metadata API failed");
                    break;
            }
        }];
        
        [self waitForExpectationsWithTimeout:ExpectationTimeOut*2 handler:^(NSError *error) {
            
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
        }];
        
    } else {
        XCTFail(@"Test Failed due to nil access token");
    }
}

/*!
 * Test Case: If response code is 200,  move API response contains the file which moved.
 */
- (void)testToMoveFileToTarget {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        XCTestExpectation *expectation = [self expectationWithDescription:@"Test to move file to given target"];
        
        PCMetaDataAPIParams *params = [[PCMetaDataAPIParams alloc] init];
        params.path = @"/VZMOBILE";
        
        [[PCAPISDK sharedInstance] fetchMetadata:params withCompletionHandler:^(PCMetadata *metadata, PCAPIQueryResponse *response) {
            switch (response.urlResponse.statusCode) {
                case PCAPIResponseCodeOK: {
                    PCFOPSAPIParams *params = [[PCFOPSAPIParams alloc] init];
                    PCResource *resource = [metadata.subFiles objectAtIndex:0];
                    params.source = resource.fullPath;
                    
                    if ([metadata.subFolders count]) {
                        PCMetadata *metadataTarget = [metadata.subFolders objectAtIndex:0];
                        params.target = metadataTarget.fullPath;
                    }
                    
                    
                    [[PCAPISDK sharedInstance] move:params withCompletionHandler:^(PCResource *metadata, PCAPIQueryResponse *response) {
                        
                        switch (response.urlResponse.statusCode) {
                            case PCAPIResponseCodeOK: {
                                PCMetaDataAPIParams *params = [[PCMetaDataAPIParams alloc] init];
                                params.path = metadata.fullPath;
                                
                                [[PCAPISDK sharedInstance] fetchMetadata:params withCompletionHandler:^(PCMetadata *metadata, PCAPIQueryResponse *response) {
                                    
                                    switch (response.urlResponse.statusCode) {
                                        case PCAPIResponseCodeOK:{
                                            BOOL isContain = NO;
                                            for (PCResource *moveResource in metadata.subFiles) {
                                                
                                                if ([moveResource.name isEqualToString:resource.name]) {
                                                    isContain = YES;
                                                }
                                            }
                                            if (!isContain) {
                                                XCTFail(@"Test could not be completed because move failed");
                                            }
                                        }
                                            break;
                                            
                                        default:
                                            XCTFail(@"Test could not be completed because metadata API failed");
                                            break;
                                    }
                                    
                                    [expectation fulfill];
                                }];
                                
                            }
                                break;
                            default:
                                XCTFail(@"Test could not be completed because move API failed");
                                break;
                        }
                        
                    }];
                    
                }
                    break;
                    
                default:
                    XCTFail(@"Test could not be completed because metadata API failed");
                    break;
            }
        }];
        
        [self waitForExpectationsWithTimeout:ExpectationTimeOut*3 handler:^(NSError *error) {
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
        }];
        
    } else {
        
        XCTFail(@"Test Failed due to nil access token");
        
    }
    
}


@end
