//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import "PCResourceAPIParams.h"

/*!
 * This category has the helper methods to create a search query string.
 */
@interface NSString (SearchQueryHelperMethods)

/*!
 * Call this method to construct a keyvalue pair in the form key:value.
 * @param key key  for eg. contentType.
 * @param value value for eg. image/png.
 */
+ (NSString *)setValue:(NSString *)value forKey:(NSString *)key;


/*!
 * Call this method to a AND over N number of subquery.
 * @param args an array of keyvalue pairs on which AND operation needs to be performed.
 */
+ (NSString *)doAND:(NSArray *)args;

/*!
 * Call this method to a OR over N number of subquery.
 * @param args an array of keyvalue pairs on which OR operation needs to be performed.
 */
+ (NSString *)doOR:(NSArray *)args;

@end


/*!
 * PCSearchAPIParams stores all the parameters required to do a search in Verizon cloud. It also has the logic to do parameter validation.
 */
@interface PCSearchAPIParams : PCResourceAPIParams

/*!
 * Contains the text to be searched and supports a number of query fields that allow clients to restrict the search to particular parts of a file's or folder's metadata. The query parameter is based on Lucene search query syntax which takes the following format: ￼￼￼￼query=field:value AND field:value AND field:value. Search can be done on the following fields. author, commentCount, contentChecksum, contentType, extension, file(Set to true to search for files only and false to search for folders only), name, path, size, versionCreated. (Required)
 */
@property (nonatomic, copy) NSString *query;

/*!
 * Specify sort order for response. Syntax is :'{field}+{asc|desc}'. Valid values for 'field' are: name, type, versionCreated, size, extension, album, artist, captureDate, compilation, contentType, creationDate, favorite, genre, height, modificationDate, priority, source, tags, title, timelineDate.
 */
@property (nonatomic, copy, readonly) NSString *sort;

/*!
 * Starting point for partial responses, for folder requests. Default is 1.
 */
@property (nonatomic, assign) int start;

/*!
 * Maximum items to include in a paginated response, for folder requests. Required if start is specified.
 */
@property (nonatomic, assign) int count;

/*!
 * Call this method to set sort order for search results.
 * \param sortKey the field on which sort has to be done.
 * \param ascending if true, results come in ascending order else result comes in descending order.
 */
- (void)setSortKey:(NSString *)sortKey isAscending:(BOOL)ascending;

@end
